"""
Subclasses of grok.MultiAdapter must declare what they adapt, using grok.adapts:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: <class 'grok.tests.adapter.multiadaptsnone.Home'> must specify
  which contexts it adapts (use the 'adapts' directive to specify).
"""
import grok

from zope import interface

class IHome(interface.Interface):
    pass

class Home(grok.MultiAdapter):
    grok.implements(IHome)



##############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok directives.

This module defines Grok directives: the markers that users place inside
of their classes (and sometimes in their modules, too) to direct how
Grok registers their components.  For example, the first directive
defined below is `local_utility`, which people programming Grok
applications normally use like this::

    class MyUtility(grok.Utility):
        grok.local_utility()
        ...

If the set of directives in this module looks rather small, remember
that most of the directives available in Grok actually come from the
`grokcore` modules on which Grok depends, where they have been placed so
that other projects can use them without having to pull in all of Grok.

"""
import grok
from zope import interface
from zope.interface.interfaces import IInterface

import martian
from martian import util
from martian.error import GrokImportError
from grokcore.view.directive import TaggedValueStoreOnce

class local_utility(martian.Directive):
    """The `grok.local_utility()` directive.

    Place this directive inside of a `grok.Application` or `grok.Site`
    subclass, and provide the name of a utility you want activated
    inside of that site::

        class MySite(grok.Site):
            grok.local_utility(MyMammothUtility)
            ...

    This directive can be supplied several times within the same site.
    Thanks to the presence of this directive, any time an instance of
    your class is created in the Zope database it will have a copy of
    the given local utility installed along with it.

    This directive accepts several normal Component-registration keyword
    arguments, like `provides` and `name`, and uses them each time it
    registers your local utility.

    If you do not supply a `provides` keyword, then Grok attempts to
    guess a sensible default.  Its first choice is to use any
    interface(s) that you listed with the grok.provides() directive when
    defining your utility.  Otherwise, if your utility is a subclass of
    `grok.localUtility`, then Grok will use any interfaces that your
    utility supplies beyond those are supplied because of its
    inheritance from `grok.localUtility`.  Else, as a final fallback, it
    checks to see whether the class you are registering supplies one,
    and only one, interface; if so, then it can register the utility
    unambiguously as providing that one interface.

    """
    scope = martian.CLASS
    store = martian.DICT

    def factory(self, factory, provides=None, name=u'',
                setup=None, public=False, name_in_container=None):
        if provides is not None and not IInterface.providedBy(provides):
            raise GrokImportError("You can only pass an interface to the "
                                  "provides argument of %s." % self.name)

        if provides is None:
            provides = grok.provides.bind().get(factory)

        if provides is None:
            if util.check_subclass(factory, grok.LocalUtility):
                baseInterfaces = interface.implementedBy(grok.LocalUtility)
                utilityInterfaces = interface.implementedBy(factory)
                provides = list(utilityInterfaces - baseInterfaces)

                if len(provides) == 0 and len(list(utilityInterfaces)) > 0:
                    raise GrokImportError(
                        "Cannot determine which interface to use "
                        "for utility registration of %r. "
                        "It implements an interface that is a specialization "
                        "of an interface implemented by grok.LocalUtility. "
                        "Specify the interface by either using grok.provides "
                        "on the utility or passing 'provides' to "
                        "grok.local_utility." % factory, factory)
            else:
                provides = list(interface.implementedBy(factory))

            util.check_implements_one_from_list(provides, factory)
            provides = provides[0]

        if (provides, name) in self.frame.f_locals.get(self.dotted_name(), {}):
            raise GrokImportError(
                "Conflicting local utility registration %r. "
                "Local utilities are registered multiple "
                "times for interface %r and name %r." %
                (factory, provides, name), factory)

        info = LocalUtilityInfo(factory, provides, name, setup, public,
                                name_in_container)
        return (provides, name), info


class LocalUtilityInfo(object):
    """The information about how to register a local utility.

    An instance of this class is created for each `grok.local_utility()`
    in a Grok application's code, to remember how the user wants their
    local utility registered.  Later, whenever the application creates
    new instances of the site or application for which the local utility
    directive was supplied, this block of information is used as the
    parameters to the creation of the local utility which is created
    along with the new site in the Zope database.

    """
    _order = 0

    def __init__(self, factory, provides, name=u'',
                 setup=None, public=False, name_in_container=None):
        self.factory = factory
        self.provides = provides
        self.name = name
        self.setup = setup
        self.public = public
        self.name_in_container = name_in_container

        self.order = LocalUtilityInfo._order
        LocalUtilityInfo._order += 1

    def __cmp__(self, other):
        # LocalUtilityInfos have an inherit sort order by which the
        # registrations take place.
        return cmp(self.order, other.order)


class site(martian.Directive):
    """The `grok.site()` directive.

    This directive is used when creating a `grok.Indexes` subclass, to
    indicate the Grok site object for which the indexes should be built.

    """
    scope = martian.CLASS
    store = martian.ONCE
    validate = martian.validateInterfaceOrClass

class permissions(martian.Directive):
    """The `grok.permissions()` directive.

    This directive is used inside of a `grok.Role` subclass to list the
    permissions which each member of the role should always possess.
    Note that permissions should be passed as strings, and that several
    permissions they can simply be supplied as multiple arguments; there
    is no need to place them inside of a tuple or list::

        class MyRole(grok.Role):
            grok.permissions('page.CreatePage', 'page.EditPage')
            ...

    """
    scope = martian.CLASS
    store = martian.ONCE
    default = []

    def factory(self, *args):
        return args

class traversable(martian.Directive):
    """The `grok.traversable()` directive.

    Each time this directive is used inside of a class, it designates an
    attribute of that class which URLs should be able to traverse.  For
    example, the declaration:

        class Mammoth(grok.Model):
            grok.traversable('thighbone')

    means that if the URL `/app/mymammoth` designates a Mammoth, then
    `/app/mymammoth/thighbone` will also be a valid URL (assuming that
    the Mammoth instance, at runtime, indeed has an attribute by that
    name)!  By default, the name that must be appended to the URL should
    simply be the same as the name of the attribute; but by providing a
    `name` keyword argument, the programmer can designate another name
    to appear in the URL instead of the raw attribute name.

    """
    scope = martian.CLASS
    store = martian.DICT

    def factory(self, attr, name=None):
        if name is None:
            name = attr
        return (name, attr)

class restskin(martian.Directive):
    """The `grok.restskin()` directive.

    This directive is placed inside of `grok.IRESTLayer` subclasses to
    indicate what their layer name will be within a REST URL.  Giving
    the skin ``grok.restskin('b')``, for example, will enable URLs that
    look something like `http://localhost/++rest++b/app`.

    """
    # We cannot do any better than to check for a class scope. Ideally we
    # would've checked whether the context is indeed an Interface class.
    scope = martian.CLASS
    store = TaggedValueStoreOnce()
    validate = martian.validateText

    def factory(self, value=None):
        return value



"""
A site can be created by mixing in grok.Site into a grok.Model or
grok.Container.

  >>> from zope import interface
  >>> from zope.app.component.interfaces import IPossibleSite, ISite
  >>> manfred = Mammoth()
  >>> IPossibleSite.providedBy(manfred)
  True
  >>> herd = Herd()
  >>> IPossibleSite.providedBy(herd)
  True
  >>> nonsite = NonSite()
  >>> IPossibleSite.providedBy(nonsite)
  False
  >>> nonsitecontainer = NonSiteContainer()
  >>> IPossibleSite.providedBy(nonsitecontainer)
  False

While manfred and herd are possible sites, they are not yet sites;

  >>> ISite.providedBy(manfred)
  False
  >>> ISite.providedBy(herd)
  False
  
When a site is added to a container it will be initialized as a site
(when the ObjectAddedEvent is fired):

  >>> nonsitecontainer['manfred'] = manfred
  >>> ISite.providedBy(manfred)
  True
  >>> nonsitecontainer['herd'] = herd
  >>> ISite.providedBy(herd)
  True
"""
import grok

class Mammoth(grok.Model, grok.Site):
    pass

class Herd(grok.Container, grok.Site):
    pass

class NonSite(grok.Model):
    pass

class NonSiteContainer(grok.Container):
    pass



"""
If no model can be found in the module, we get an error:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: No module-level context for
  <class 'grok.tests.adapter.nomodel.Home'>, please use the 'context' directive.

"""
import grok
from zope import interface

class IHome(interface.Interface):
    pass

class Home(grok.Adapter):
    grok.implements(IHome)



import grok
from datetime import datetime

class Sample(grok.Application, grok.Container):
    pass

class Index(grok.View):    
    def current_datetime(self):
        now = datetime.now()
        return now.strftime('%Y-%m-%d %H:%M')



import grok

grok.templatedir('calc_templates')  # this is actually the default (from module name, calc.py)

class Calculator(grok.Model):
    pass

class Sum(grok.View):
    """Simple view for a model"""
    grok.context(Calculator)  # this is actually the default (from module)
    grok.template('sum')  # this is actually the default (from class name)
    grok.name('sum')  # this is actually the default (from class name)
    grok.require('zope.Public')  # this is actually the default

    def calculateSum(self):
        """you can pull this in the template through view/calculateSum"""

    def update(self):    
        """executed before the template is rendered"""
        self.sum = self.calculateSum()
        self.sendEmail()

    def sendEmail(self):
        """send an email here"""

class PDFSum(grok.View):

    def update(self):
        pass

    def render(self):
        return pdfdata

sum = grok.PageTemplate("""\
<p tal:content="view/calculateSum">...</p>
<p tal:content="view/precalculatedSum">...</p>
""")


from zope import schema

class Index(grok.Form):
    """a form

    this is the default view for the Calculator model (because it's
    called Index)
    """

    class fields:
        operand = schema.Int(title=u'Operand')
        operator = schema.Choice(...)
        operand2 = schema.Int(...)

    @grok.action('Calculate')
    def calculate(self, operand, operator, operand2):
        """it's possible to receive any number of form fields in any
        order, or just use **kw to receive them all in a dict"""
        self.result = operator(operand, operand2)

    # this will raise a helpful error message at startup time (encoded
    # strings)
    @grok.action('Bلللل')
    def whatever(self, **data):
        pass

index = grok.PageTemplate("""\
<form tal:attributes="action request/URL">
<p tal:condition="exists:view/result">
  The result is: <span tal:replace="view/result" />
</p>

XXX render fields
XXX render actions
</form>
""")

class CalculatorXMLRPC(grok.XMLRPC):

    @grok.require('zope.Public')  # this is actually the default
    def sum(self, operand, operator, operand2):
        return ...

    @grok.require('something.else')
    def whatever(self):
        return ...



"""
REST objects, like all views, are properly located objects and will
therefore honour local grants, for instance.  Let's consider the
following model in the root folder.

  >>> root = getRootFolder()
  >>> root['manfred'] = manfred = Mammoth('manfred')

For this model we have registered a REST GET view that's protected
with a permission.  Therefore we can't access it as anonymous:

  >>> print http(r'''
  ... GET /++rest++mammoth/manfred HTTP/1.1
  ... ''')
  HTTP/1.1 401 Unauthorized
  Content-Length: 0
  Content-Type: text/plain
  WWW-Authenticate: basic realm="Zope"

However, if we make a (local!) grant, e.g. on the root object, we can
access the view just fine:

  >>> from zope.securitypolicy.interfaces import IPrincipalPermissionManager
  >>> root_perms = IPrincipalPermissionManager(root)
  >>> root_perms.grantPermissionToPrincipal('mammoth.Touch', 'zope.anybody')

With the grant in place we can access it as anonymous:

  >>> print http(r'''
  ... GET /++rest++mammoth/manfred HTTP/1.1
  ... ''')
  HTTP/1.1 200 Ok
  Content-Length: 7
  Content-Type: text/plain
  <BLANKLINE>
  manfred

In fact, inspecting the view object itself, we see that it is a true
ILocation and has the appropriate parent pointer:

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest(skin=MammothRestLayer)
  >>> from zope.component import getMultiAdapter
  >>> view = getMultiAdapter((manfred, request), name='GET')

  >>> from zope.location.interfaces import ILocation
  >>> ILocation.providedBy(view)
  True
  >>> view.__parent__ is manfred
  True

"""
import grok

class Mammoth(grok.Model):

    def __init__(self, name):
        self.name = name

class MammothRestLayer(grok.IRESTLayer):
    grok.restskin('mammoth')

class TouchMammoth(grok.Permission):
    grok.name('mammoth.Touch')

class MammothRest(grok.REST):
    grok.layer(MammothRestLayer)

    @grok.require(TouchMammoth)
    def GET(self):
        return self.context.name



"""
This is similar to catalog.py, except that the site uses a base class
which also defines a local utility.

Let's setup a site in which we manage a couple of objects:

  >>> herd = Herd()
  >>> getRootFolder()['herd'] = herd
  >>> from zope.app.component.hooks import setSite
  >>> setSite(herd)

Now we add some indexable objects to the site:

  >>> herd['manfred'] = Mammoth('Manfred')
  >>> herd['ellie'] = Mammoth('Ellie')

Then we are able to query the catalog:

  >>> from zope.app.catalog.interfaces import ICatalog
  >>> from zope.component import getUtility
  >>> catalog = getUtility(ICatalog)
  >>> for obj in catalog.searchResults(name=('Ellie', 'Ellie')):
  ...     print obj.name
  Ellie

"""

import grok
from zope import schema, interface
from zope.app.intid import IntIds
from zope.app.intid.interfaces import IIntIds
from zope.app.catalog.catalog import Catalog
from zope.app.catalog.interfaces import ICatalog
from zope.app.catalog.field import FieldIndex

def setup_catalog(catalog):
    catalog['name'] = FieldIndex('name', IMammoth)

class IMammoth(interface.Interface):

    name = schema.TextLine()

class Mammoth(grok.Model):
    grok.implements(IMammoth)

    def __init__(self, name):
        self.name = name

class BaseHerd(grok.Container, grok.Site):
    grok.local_utility(IntIds, provides=IIntIds)

class Herd(BaseHerd):
    grok.local_utility(Catalog, provides=ICatalog, setup=setup_catalog)



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""LaTeX hacks for sphinx problems.

The sphinx latextranslator currently lacks a few features that are
part of stock ``docutils`` translators. Some of the monkey patches in
here were already applied to the stock sphinx package, but some are
still missing. Therefore we 'inject' those changes here.
"""

from docutils import nodes
from sphinx.latexwriter import LaTeXTranslator

# Inject a working pygments workaround.
def depart_literal_block(self, node):
    hlcode = self.highlighter.highlight_block(self.verbatim.rstrip(
        '\n'), self.highlightlang)
    # workaround for Unicode issue
    hlcode = hlcode.replace(u'€', u'@texteuro[]')
    # workaround for Pygments bug
    hlcode = hlcode.replace('\\end{Verbatim}',
                            '\n\\end{Verbatim}')
    self.body.append('\n' + hlcode)
    self.verbatim = None
LaTeXTranslator.depart_literal_block = depart_literal_block
LaTeXTranslator.depart_doctest_block = depart_literal_block

# Inject a topic handler that frames sidebars and topics with a
# shadowbox instead of the normal fbox.
def visit_topic(self, node):
    self.body.append('\\setbox0\\vbox{\n'
                     '\\begin{minipage}{0.75\\textwidth}\n')
def depart_topic(self, node):
    self.body.append('\\end{minipage}}\n'
                     '\\begin{center}\\setlength{\\fboxsep}{5pt}'
                     '\\shadowbox{\\box0}\\end{center}\n')
LaTeXTranslator.visit_topic = visit_topic
LaTeXTranslator.depart_topic = depart_topic
LaTeXTranslator.visit_sidebar = visit_topic
LaTeXTranslator.depart_sidebar = depart_topic

def visit_image(self, node):
    attrs = node.attributes
    # Add image URI to dependency list, assuming that it's
    # referring to a local file.
    #self.settings.record_dependencies.add(attrs['uri'])
    pre = []                        # in reverse order
    post = []
    include_graphics_options = ""
    inline = isinstance(node.parent, nodes.TextElement)
    if attrs.has_key('scale'):
        # Could also be done with ``scale`` option to
        # ``\includegraphics``; doing it this way for consistency.
        pre.append('\\scalebox{%f}{' % (attrs['scale'] / 100.0,))
        post.append('}')
    if attrs.has_key('width'):
        include_graphics_options = '[width=%s]' % attrs['width']
    if attrs.has_key('align'):
        align_prepost = {
            # By default latex aligns the top of an image.
            (1, 'top'): ('', ''),
            (1, 'middle'): ('\\raisebox{-0.5\\height}{', '}'),
            (1, 'bottom'): ('\\raisebox{-\\height}{', '}'),
            (0, 'center'): ('{\\hfill', '\\hfill}'),
            # These 2 don't exactly do the right thing.
            # The image should be floated alongside the
            # paragraph.  See
            # http://www.w3.org/TR/html4/struct/objects.html#adef-align-IMG
            (0, 'left'): ('{', '\\hfill}'),
            (0, 'right'): ('{\\hfill', '}'),}
        try:
            pre.append(align_prepost[inline, attrs['align']][0])
            post.append(align_prepost[inline, attrs['align']][1])
        except KeyError:
            pass                    # XXX complain here?
    if not inline:
        pre.append('\n')
        post.append('\n')
        pre.reverse()
        self.body.extend( pre )
        self.body.append( '\\includegraphics%s{%s}' % (
            include_graphics_options, attrs['uri'] ) )
        self.body.extend( post )

def depart_image(self, node):
    pass

LaTeXTranslator.visit_image = visit_image
LaTeXTranslator.depart_image = depart_image




"""
We cannot register two rest protocols under the same name::

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  ConfigurationConflictError: Conflicting configuration actions
    For: ('restprotocol', 'foo')
"""

import grok

class Protocol1(grok.IRESTLayer):
    grok.restskin('foo')

class Protocol2(grok.IRESTLayer):
    grok.restskin('foo')



"""
The MultipleTimesAsDictDirective is used by grok.traversable so multiple
attributes can be mentioned.

  >>> from martian import scan
  >>> import grok
  >>> g = grok.traversable.bind().get(Club)
  >>> isinstance(g, dict)
  True
  >>> g['demo']
  'demo'
  >>> g['attr']
  'attr'
  >>> g['asdf']
  'attr'
"""
import grok
from zope import interface

class Club(grok.Model):
    grok.traversable('asdf', name='attr')
    grok.traversable('attr')
    grok.traversable('attr', name='asdf')
    grok.traversable('demo')
    demo = 'something'



"""
Grok allows you to set up catalog indexes in your application with a
special indexes declaration.

Let's set up a site in which we manage a couple of objects::

  >>> herd = Herd()
  >>> getRootFolder()['herd'] = herd
  >>> from zope.app.component.hooks import setSite
  >>> setSite(herd)

Now we add some indexable objects to the site::

  >>> herd['alpha'] = Mammoth('Alpha', 13, 'Hello world!')
  >>> herd['beta'] = Mammoth('Beta', 14, 'Bye World!')

We are able to query the catalog::

  >>> from zope.app.catalog.interfaces import ICatalog
  >>> from zope.component import getUtility, queryUtility
  >>> catalog = getUtility(ICatalog)
  >>> for obj in catalog.searchResults(name=('Beta', 'Beta')):
  ...   print obj.name
  Beta

Let's query the text index, which incidentally also indexes a method::

  >>> def sortedResults(catalog, **kw):
  ...    result = list(catalog.searchResults(**kw))
  ...    result.sort(key=lambda x:x.name)
  ...    return [item.name for item in result]
  >>> sortedResults(catalog, message='world')
  ['Alpha', 'Beta']
  >>> sortedResults(catalog, message='hello')
  ['Alpha']
  >>> sortedResults(catalog, message='bye')
  ['Beta']

Note that another application that we did not register the
indexes for won't have a catalog available::

  >>> herd2 = Herd2()
  >>> getRootFolder()['herd2'] = herd2
  >>> setSite(herd2)
  >>> queryUtility(ICatalog, default=None) is None
  True
  >>> setSite(herd)
  
Nuke the catalog and intids in the end, so as not to confuse
other tests::

  >>> sm = herd.getSiteManager()
  >>> from zope.app.catalog.interfaces import ICatalog
  >>> sm.unregisterUtility(catalog, provided=ICatalog)
  True
  >>> from zope.app.intid.interfaces import IIntIds
  >>> from zope import component
  >>> intids = component.getUtility(IIntIds)
  >>> sm.unregisterUtility(intids, provided=IIntIds)
  True

Unfortunately ftests don't have good isolation from each other yet.
"""

from zope.interface import Interface
from zope import schema

import grok
from grok import index

class Herd(grok.Container, grok.Application):
    pass

class Herd2(grok.Container, grok.Application):
    pass

class IMammoth(Interface):
    name = schema.TextLine(title=u'Name')
    age = schema.Int(title=u'Age')
    def message():
        """Message the mammoth has for the world."""

class MammothIndexes(grok.Indexes):
    grok.site(Herd)
    grok.context(IMammoth)

    name = index.Field()
    age = index.Field()
    message = index.Text()

class Mammoth(grok.Model):
    grok.implements(IMammoth)

    def __init__(self, name, age, message):
        self.name = name
        self.age = age
        self._message = message

    def message(self):
        return self._message
    



"""
Base classes shouldn't be grokked. The way is to use the
'grok.baseclass' directive on the class itself.

  >>> grok.testing.grok(__name__)

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope import component
  >>> model = ModelBase()

  >>> view = component.getMultiAdapter((model, request), name='anotherview')
  Traceback (most recent call last):
    ...
  ComponentLookupError: ((<grok.tests.baseclass.base.ModelBase object at 0x...>,
  <zope.publisher.browser.TestRequest instance ...>),
  <InterfaceClass zope.interface.Interface>,
  'anotherview')
  
"""
import grok

class ModelBase(grok.Model):
    pass

class AnotherView(grok.View):
    grok.baseclass()
    
    def render(self):
        return "hello world"



"""
A Grokker can declare that scanning should continue, so that other Grokkers can
still perform actions on the grokked components.

Here we define AlphaGrokker which has higher priority than BetaGrokker but does
not block BetaGrokker from picking up the same component::

    >>> import grok
    >>> grok.testing.grok(__name__)

In the fixture there is AlphaBetaSub that inherits from both Alpha and Beta.
Thus, both Grokkers are executed, with AlphaGrokker coming before BetaGrokker::

    >>> grok.testing.grok('grok.tests.grokker.continue_scanning_fixture')
    alpha
    beta

"""
import martian


class Alpha(object):
    pass

class Beta(object):
    pass

class AlphaGrokker(martian.ClassGrokker):
    martian.component(Alpha)
    martian.priority(1) # we need to go before BetaGrokker

    def grok(self, name, factory, module_info, config, **kw):
        print "alpha"
        return True

class BetaGrokker(martian.ClassGrokker):
    martian.component(Beta)

    def grok(self, name, factory, module_info, config, **kw):
        print "beta"
        return True
    



"""
@grok.subscribe can only be used on module level:

  >>> function_context()
  Traceback (most recent call last):
    ...
  GrokImportError: @grok.subscribe can only be used on module level.
  
  >>> class_context()
  Traceback (most recent call last):
    ...
  GrokImportError: @grok.subscribe can only be used on module level.


@grok.subscribe can not be called without arguments:

  >>> import grok.tests.event.errorconditions_fixture
  Traceback (most recent call last):
    ...
  GrokImportError: @grok.subscribe requires at least one argument.
  
"""
import grok

def function_context():
    @grok.subscribe(grok.Model, grok.IObjectCreatedEvent)
    def subscriber():
        pass
    
def class_context():
    class Wrapper:
        @grok.subscribe(grok.Model, grok.IObjectCreatedEvent)
        def subscriber(self):
            pass



"""
  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: <class 'grok.tests.json.nomethods.RemoteCaveman'> does not
  define any public methods. Please add methods to this class to enable
  its registration.

"""
import grok

class Caveman(grok.Model):
    pass

class RemoteCaveman(grok.JSON):
    pass



"""
Subclassed sites inherit all local utilities of their base classes:

  >>> cave = BigCave()
  >>> getRootFolder()["cave"] = cave

  >>> from zope import component
  >>> from zope.app.component.hooks import getSite, setSite

  >>> setSite(cave)
  >>> fireplace = component.getUtility(IFireplace)
  >>> IFireplace.providedBy(fireplace)
  True
  >>> isinstance(fireplace, Fireplace)
  True

Additional utilities can be registered in the subclass:
  
  >>> hollow = HollowCave()
  >>> getRootFolder()["hollow"] = hollow

  >>> setSite(hollow)
  >>> fireplace = component.getUtility(IFireplace)
  >>> IFireplace.providedBy(fireplace)
  True
  >>> isinstance(fireplace, Fireplace)
  True

  >>> painting = component.getUtility(IPainting)
  >>> IPainting.providedBy(painting)
  True
  >>> isinstance(painting, Painting)
  True

Those do not influence the base class:

  >>> setSite(cave)
  >>> painting = component.getUtility(IPainting)
  Traceback (most recent call last):
    ...
  ComponentLookupError: (<InterfaceClass grok.ftests.utility.subclass.IPainting>, '')

This works various levels of inheritance deep:

  >>> very_hollow = VeryHollowCave()
  >>> getRootFolder()['very_hollow'] = very_hollow

  >>> setSite(very_hollow)
  >>> fireplace = component.getUtility(IFireplace)
  >>> painting = component.getUtility(IPainting)
  >>> great_painting = component.getUtility(IPainting, 'great')
  >>> bad_painting = component.getUtility(IPainting, 'bad')

And with inheritance hierarchies where a base class is inherited multiple
times through different routes:

  >>> scary = ScaryCave()
  >>> getRootFolder()['scary'] = scary

  >>> setSite(scary)
  >>> fireplace = component.getUtility(IFireplace)
  >>> painting = component.getUtility(IPainting)
  >>> great_painting = component.getUtility(IPainting, 'great')
  >>> bad_painting = component.getUtility(IPainting, 'bad')

"""
import grok
from zope import interface

class IFireplace(interface.Interface):
    pass

class IPainting(interface.Interface):
    pass

class Fireplace(grok.LocalUtility):
    grok.implements(IFireplace)

class Painting(grok.LocalUtility):
    grok.implements(IPainting)

class Cave(grok.Model, grok.Site):
    # we use name_in_container here to prevent multiple registrations
    # since storing the utilities multiple times under the same name
    # would raise a DuplicationError
    grok.local_utility(Fireplace, name_in_container='fireplace')

class BigCave(Cave):
    pass

class HollowCave(Cave):
    grok.local_utility(Painting)

class VeryHollowCave(HollowCave):
    grok.local_utility(Painting, name='great')
    grok.local_utility(Painting, name='bad')

# this cave subclasses from Cave twice
class ScaryCave(VeryHollowCave, Cave):
    pass




import grok
from zope import interface
from BTrees.OOBTree import OOTreeSet

class Article(grok.Model):
    pass

class IComments(interface.Interface):

    def addComment(text):
        pass

    def getComments():
        pass

class Comments(grok.Annotation):
    grok.context(Article)  # this is actually the default
    grok.implements(IComments)
    grok.name('annotations.Comments')  # this is actually the default

    def __init__(self): 
        self.comments = OOTreeSet()

    def addComment(self, text):
        self.comments.insert(text)

    def getComments(self):
        return list(self.comments)



"""
Containers can also have an explicit traverser associated with them.
The behavior falls back to basic container traversal if the 'traverse'
method returns None:

  >>> getRootFolder()["herd"] = herd = Herd()
  >>> herd['manfred'] = Mammoth('Manfred')
  >>> herd['ellie'] = Mammoth('Ellie')

Let's first try to look up the special traversed item:

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/herd/special")
  >>> print browser.contents
  special view
  >>> browser.open("http://localhost/herd/special/index")
  >>> print browser.contents
  special view

Even if we have a container item called 'special', we should still
get our special object:

  >>> herd['special'] = Mammoth('Special invisible mammoth')
  >>> browser.open("http://localhost/herd/special")
  >>> print browser.contents
  special view
  >>> browser.open("http://localhost/herd/special/index")
  >>> print browser.contents
  special view
  
The fall-back behavior should work for items that aren't traversed:

  >>> browser.open("http://localhost/herd/manfred")
  >>> print browser.contents
  <html>
  <body>
  <h1>Hello, Manfred!</h1>
  </body>
  </html>

  >>> browser.open("http://localhost/herd/ellie")
  >>> print browser.contents
  <html>
  <body>
  <h1>Hello, Ellie!</h1>
  </body>
  </html>

Also try traversing (an empty and therefore False in a Boolean sense) container
as a subitem of a container:

  >>> herd['subherd'] = Herd()
  >>> browser.open("http://localhost/herd/subherd/special")
  >>> print browser.contents
  special view

"""
import grok

class Herd(grok.Container):
    pass

class Traverser(grok.Traverser):
    grok.context(Herd)
    def traverse(self, name):
        if name == 'special':
            return Special()
        return None

class Mammoth(grok.Model):
    def __init__(self, name):
        self.name = name

class Special(grok.Model):
    pass

class SpecialIndex(grok.View):
    grok.context(Special)
    grok.name('index')
    
    def render(self):
        return "special view"

grok.context(Mammoth)

class Index(grok.View):
    pass

index = grok.PageTemplate("""\
<html>
<body>
<h1>Hello, <span tal:replace="context/name/title" />!</h1>
</body>
</html>
""")



"""

After grokking a module that defines an application, the application factory is
available as a utility::

    >>> grok.testing.grok(__name__)
    >>> import zope.component
    >>> import grok.interfaces
    >>> calendar_app = zope.component.getUtility(grok.interfaces.IApplication,
    ...                                          name='grok.tests.application.application.Calendar')
    >>> calendar_app
    <class 'grok.tests.application.application.Calendar'>

Applications are both containers and sites::

    >>> issubclass(calendar_app, grok.Site)
    True

Applications can be instanciated without any arguments::

    >>> calendar = calendar_app()
    >>> calendar
    <grok.tests.application.application.Calendar object at 0x...>

"""

import grok


class Calendar(grok.Application):
    """A calendar application that knows about ancient
    calendar systems from the stone age.
    """



"""
A permission has to be defined first (using grok.Permission for
example) before it can be used in grok.require() in an XMLRPC
class. However, this is *not* the the case for a default permission
that is never used.

  >>> grok.testing.grok(__name__)

"""

import grok
import zope.interface

class Foo(grok.Permission):
    grok.name('foo')

class MissingPermission(grok.XMLRPC):
    grok.context(zope.interface.Interface)
    grok.require('doesnt.exist')

    @grok.require(Foo)
    def foo(self):
        pass



##############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""XML/RPC access to the wiki pages
"""

import grok
import grokwiki.page


class WikiPageRPC(grok.XMLRPC):
    grok.context(grokwiki.page.WikiPage)

    def edit(self, text):
        self.context.update(text)

    def show(self):
        return self.context.text



import grok

class Sample(grok.Application, grok.Container):
    def information(self):
        return "This is important information!"

class Index(grok.View):
    def reversed_information(self):
        return ''.join(reversed(self.context.information()))



##############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok
"""
from zope.interface import implements
from zope.component import adapts

from martian import ClassGrokker, InstanceGrokker, GlobalGrokker
from martian import baseclass
from martian.error import GrokError, GrokImportError

from grokcore.component import Adapter, MultiAdapter, GlobalUtility, Context
from grokcore.component.decorators import subscribe, adapter, implementer
from grokcore.component.directive import (
    context, name, title, description, provides, global_utility, direct)

from grokcore.security import Permission
from grokcore.security import Public
from grokcore.security import require

from grokcore.view import PageTemplate
from grokcore.view import PageTemplateFile
from grokcore.view import DirectoryResource
from grokcore.view import layer
from grokcore.view import template
from grokcore.view import templatedir
from grokcore.view import skin
from grokcore.view import url
from grokcore.view import path

from grokcore.viewlet import Viewlet
from grokcore.viewlet import ViewletManager
from grokcore.viewlet import view
from grokcore.viewlet import viewletmanager
from grokcore.viewlet import order

from grokcore.formlib import action
from grokcore.formlib import AutoFields
from grokcore.formlib import Fields

from zope.event import notify
from zope.app.component.hooks import getSite
from zope.lifecycleevent import (
    IObjectCreatedEvent, ObjectCreatedEvent,
    IObjectModifiedEvent, ObjectModifiedEvent,
    IObjectCopiedEvent, ObjectCopiedEvent)
from zope.app.publication.interfaces import IBeforeTraverseEvent

from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

from zope.app.container.contained import (
    IObjectAddedEvent, ObjectAddedEvent,
    IObjectMovedEvent, ObjectMovedEvent,
    IObjectRemovedEvent, ObjectRemovedEvent,
    IContainerModifiedEvent, ContainerModifiedEvent)

from grok.components import Model, View
from grok.components import XMLRPC, REST, JSON
from grok.components import Traverser
from grok.components import Container, OrderedContainer
from grok.components import Site, LocalUtility, Annotation
from grok.components import Application, Form, AddForm, EditForm, DisplayForm
from grok.components import Indexes
from grok.components import Role
from grok.interfaces import IRESTSkinType, IRESTLayer

from grok.directive import (
    local_utility, permissions, site, restskin, traversable)

# BBB These two functions are meant for test fixtures and should be
# imported from grok.testing, not from grok.
from grok.testing import grok, grok_component

# Our __init__ provides the grok API directly so using 'import grok' is enough.
from grok.interfaces import IGrokAPI
from zope.interface import moduleProvides
moduleProvides(IGrokAPI)
__all__ = list(IGrokAPI)



import grok
from zope import interface


grok.definelayer('my')
grok.defineskin('my')                 # Picks up the layer 'my' if it exists

grok.layer('my')                      # If there is only a single layer defined
                                      # in a module, it will be the default


class Painting(grok.View):
    pass


fireplace = grok.PageTemplate("""\
<html><body></body></html>
""")



"""
You can't call grok.context from a function:

  >>> func()
  Traceback (most recent call last):
    ...
  GrokImportError: The 'context' directive can only be used on class or
  module level.

You can't call grok.context from a method either:

  >>> SomeClass().meth()
  Traceback (most recent call last):
    ...
  GrokImportError: The 'context' directive can only be used on class or
  module level.

"""
import grok
from grok.tests.adapter.adapter import Cave

def func():
    """We don't allow calling `grok.context` from anything else than a
    module or a class"""
    grok.context(Cave)

class SomeClass(object):

    def meth(self):
        """We don't allow calling `grok.context` from anything else
        than a module or a class"""
        grok.context(Cave)



"""
Multiple models lead to ambiguity:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: Multiple possible contexts for
  <class 'grok.tests.adapter.multiple.Home'>, please use the
  'context' directive.

"""
import grok
from zope import interface

class Cave(grok.Model):
    pass

class Club(grok.Model):
    pass

class IHome(interface.Interface):
    pass

class Home(grok.Adapter):
    grok.implements(IHome)



"""
Let's examine Grok's REST support in the context of custom traversal to
verify that works. We set the REST protocol in the code instead of specifying
it in the URL, just in time (when we traverse to the content object that
supports REST).

Let's create a simple application with REST support::

  >>> from grok.ftests.rest.rest_traverse import MyApp
  >>> root = getRootFolder()
  >>> root['app'] = MyApp()

Let's first look at the index view of the application object::

  >>> response = http_call('GET', 'http://localhost/app')
  >>> print response.getBody()
  The index view

Now let's look at the content subobject, which should use REST by default
because we make sure we set the skin::

  >>> response = http_call('GET', 'http://localhost/app/content')
  >>> print response.getBody()
  GET content

The other methods should also work::

  >>> response = http_call('POST', 'http://localhost/app/content')
  >>> print response.getBody()
  POST content

  >>> response = http_call('PUT', 'http://localhost/app/content')
  >>> print response.getBody()
  PUT content

  >>> response = http_call('DELETE', 'http://localhost/app/content')
  >>> print response.getBody()
  DELETE content

Besides the Traverser, we also want to make sure our ``traverse`` method
on content objects works::

  >>> response = http_call('GET', 'http://localhost/app/content/sub')
  >>> print response.getBody()
  GET content
  >>> response = http_call('PUT', 'http://localhost/app/content/sub')
  >>> print response.getBody()
  PUT content

"""

import grok
from zope.interface import Interface
from grok.util import applySkin

class IFoo(Interface):
    pass

class MyApp(grok.Model, grok.Application):
    pass

class Traverser(grok.Traverser):
    grok.context(MyApp)

    def traverse(self, name):
        if name == 'content':
            applySkin(self.request, LayerZ, grok.IRESTSkinType)
            return MyContent()

class Index(grok.View):
    grok.context(MyApp)
    def render(self):
        return "The index view"

class MyContent(grok.Model):
    def traverse(self, name):
        if 'sub':
            return MyContent()

class LayerZ(grok.IRESTLayer):
    grok.restskin('layerz')

class ZContentRest(grok.REST):
    grok.layer(LayerZ)
    grok.context(MyContent)

    def GET(self):
        return "GET content"

    def POST(self):
        return "POST content"

    def PUT(self):
        return "PUT content"

    def DELETE(self):
        return "DELETE content"



import grok

class Sample(grok.Application, grok.Container):
    text = 'default text'

class Index(grok.View):
    pass

class Edit(grok.View):
    def update(self, text=None):
        if text is None:
            return
        self.context.text = text
        self.redirect(self.url('index'))



"""
Grok allows you to set up catalog indexes in your application with a
special indexes declaration. In fact, we have multiple grok.Indexes
setting up more than one set of indexes in the same catalog. What if these
indexes define the same names?

Let's set up a site in which we manage a couple of objects::

  >>> herd = Herd()
  >>> getRootFolder()['herd'] = herd
  Traceback (most recent call last):
    ...
  GrokError: grok.Indexes in module <module
  'grok.ftests.catalog.indexes_multiple_conflict' from ...>
  causes creation of catalog index 'name' in catalog u'', but an index
  with that name is already present.

  >>> from zope.app.component.hooks import setSite
  >>> setSite(herd)
  >>> from zope.app.catalog.interfaces import ICatalog
  >>> from zope.component import getUtility, queryUtility
  >>> catalog = getUtility(ICatalog)

Nuke the catalog and intids in the end, so as not to confuse
other tests::

  >>> sm = herd.getSiteManager()
  >>> from zope.app.catalog.interfaces import ICatalog
  >>> sm.unregisterUtility(catalog, provided=ICatalog)
  True
  >>> from zope.app.intid.interfaces import IIntIds
  >>> from zope import component
  >>> intids = component.getUtility(IIntIds)
  >>> sm.unregisterUtility(intids, provided=IIntIds)
  True

Unfortunately ftests don't have good isolation from each other yet.
"""

from zope.interface import Interface
from zope import schema

import grok
from grok import index

class Herd(grok.Container, grok.Application):
    pass

class IMammoth(Interface):
    name = schema.TextLine(title=u'Name')

class IMammoth2(Interface):
    name = schema.TextLine(title=u'Name')

class MammothIndexes(grok.Indexes):
    grok.site(Herd)
    grok.context(IMammoth)

    name = index.Field()

class MammothIndexes2(grok.Indexes):
    grok.site(Herd)
    grok.context(IMammoth2)

    name = index.Field()



"""Default REST view for Grok.

The views provided by this module get invoked when an object receives an
HTTP request in a REST skin for which no more-specific REST behavior has
been defined.  These all return the HTTP response Method Not Allowed.

"""
import grok
from grok.interfaces import IRESTSkinType

from zope import component
from zope.component.interfaces import ComponentLookupError
from zope.traversing.interfaces import TraversalError
from zope.traversing.namespace import view
from zope.interface import Interface
from zope.publisher.interfaces.http import IHTTPRequest
from zope.app.publication.http import MethodNotAllowed
from zope.publisher.browser import applySkin


class GrokMethodNotAllowed(MethodNotAllowed):
    """Exception indicating that an attempted REST method is not allowed."""


class MethodNotAllowedView(grok.MultiAdapter):
    """View rendering a REST GrokMethodNotAllowed exception over HTTP.

    Not only does this view render the REST error as an HTTP status of
    405 (Method Not Allowed) and a simple text message as the document
    body, but also offers an ``Allow:`` HTTP header listing any methods
    that can, in fact, succeed.  It constructs this list by testing the
    current object to see which methods it supports; if none of the
    standard methods succeed, then the ``Allow:`` header is still
    provided, but its value will be empty.

    """
    grok.adapts(GrokMethodNotAllowed, IHTTPRequest)
    grok.name('index.html')
    grok.implements(Interface)

    def __init__(self, error, request):
        self.error = error
        self.request = request
        self.allow = self._getAllow()

    def _getAllow(self):
        allow = []
        # List methods here in the same order that they should appear in
        # the "Allow:" header.
        for method in 'DELETE', 'GET', 'POST', 'PUT':
            view = component.queryMultiAdapter(
                (self.error.object, self.error.request),
                name=method)
            if view is not None:
                is_not_allowed = getattr(view, 'is_not_allowed', False)
                if not is_not_allowed:
                    allow.append(method)
        return allow

    def __call__(self):
        self.request.response.setHeader('Allow', ', '.join(self.allow))
        self.request.response.setStatus(405)
        return 'Method Not Allowed'


class rest_skin(view):
    """A rest skin.

    This used to be supported by zope.traversing but the change was
    backed out.  We need it for our REST support.

    """
    def traverse(self, name, ignored):
        self.request.shiftNameToApplication()
        try:
            skin = component.getUtility(IRESTSkinType, name)
        except ComponentLookupError:
            raise TraversalError("++rest++%s" % name)
        applySkin(self.request, skin)
        return self.context


class NotAllowedREST(grok.REST):
    """Default REST view, whose methods all raise Not Allowed errors.

    By binding itself to ``Interface``, this becomes the most general
    available REST view, and will be called into service for objects
    that have not had more specific REST views registered.  This means
    that such objects can at least return attractive refusals when
    clients attempt to assail them with unwanted HTTP methods.

    """
    grok.layer(grok.IRESTLayer)
    grok.context(Interface)

    is_not_allowed = True

    def GET(self):
        raise GrokMethodNotAllowed(self.context, self.request)

    def POST(self):
        raise GrokMethodNotAllowed(self.context, self.request)

    def PUT(self):
        raise GrokMethodNotAllowed(self.context, self.request)

    def DELETE(self):
        raise GrokMethodNotAllowed(self.context, self.request)



"""
  >>> grok.testing.grok(__name__)
  >>> from zope import component
  >>> from zope.annotation.attribute import AttributeAnnotations
  >>> component.provideAdapter(AttributeAnnotations)

If an annotation class implements more than one interface, it has to
declare which one it should be registered for using ``grok.provides``.

  >>> manfred = Mammoth()
  >>> ann = IOneInterface(manfred)

It can then be looked up only using that one interface:

  >>> IAnotherOne(manfred)
  Traceback (most recent call last):
  TypeError: ('Could not adapt', <grok.tests.annotation.provides.Mammoth object at ...>, <InterfaceClass grok.tests.annotation.provides.IAnotherOne>)


"""

import grok
from zope import interface
from BTrees.OOBTree import OOTreeSet

class Mammoth(grok.Model):
    pass

class IOneInterface(interface.Interface):
    pass

class IAnotherOne(interface.Interface):
    pass

class MammothAnnotation(grok.Annotation):
    grok.implements(IOneInterface, IAnotherOne)
    grok.provides(IOneInterface)



"""
Schema-driven development with grok
"""
import grok

grok.directory('contact')

class Contact(grok.SchemaModel):
    """
    This works now:

      >>> c = Contact(name=u'Martijn', city=u'Rotterdam')
      >>> c.name
      u'Martijn'

    Also w/o kw:

      >>> c = Contact(u'Martijn', u'Rotterdam')
      >>> c.name
      u'Martijn'

    """
    class fields:
        name = schema.TextLine()
        city = schema.TextLine()


class Edit(grok.EditForm):
    pass
    # this will automatically render an edit form, and use an
    # 'edit.html' template if available

class Index(grok.DisplayForm):
    pass
    # this will automatically render an display form, and use an
    # 'index.html' template if available

class Add(grok.AddForm):
    grok.context(Contact)  # this is actually the default
    grok.container(IContainer)  # this is actually the default

class FancyEdit(grok.EditForm):
    """ use cases:

    * (actions with permissions)

    *

    """

    @grok.action("Save")
    def save(self, **data):
        """
        this overrides any actions defined on the base class
        """
        self.applyChanges(data)



"""
Grok allows you to set up catalog indexes in your application with a
special indexes declaration. If you want to name the index differently
from the attribute, you can do so, by passing an explicit `attribute`
keyword argument to the field.

Let's set up a site in which we manage a couple of objects::

  >>> herd = Herd()
  >>> getRootFolder()['herd'] = herd
  >>> from zope.app.component.hooks import setSite
  >>> setSite(herd)

Now we add some indexable objects to the site::

  >>> herd['alpha'] = Mammoth('Alpha', 13)
  >>> herd['beta'] = Mammoth('Beta', 14)

We are able to query the catalog::

  >>> from zope.app.catalog.interfaces import ICatalog
  >>> from zope.component import getUtility, queryUtility
  >>> catalog = getUtility(ICatalog)
  >>> for obj in catalog.searchResults(how_old=(13, 13)):
  ...   print obj.name
  Alpha

Nuke the catalog and intids in the end, so as not to confuse
other tests::

  >>> sm = herd.getSiteManager()
  >>> from zope.app.catalog.interfaces import ICatalog
  >>> sm.unregisterUtility(catalog, provided=ICatalog)
  True
  >>> from zope.app.intid.interfaces import IIntIds
  >>> from zope import component
  >>> intids = component.getUtility(IIntIds)
  >>> sm.unregisterUtility(intids, provided=IIntIds)
  True
"""

from zope.interface import Interface
from zope import schema

import grok
from grok import index

class Herd(grok.Container, grok.Application):
    pass

class IMammoth(Interface):
    name = schema.TextLine(title=u'Name')
    age = schema.Int(title=u'Age')

class MammothIndexes(grok.Indexes):
    grok.site(Herd)
    grok.context(IMammoth)

    name = index.Field()
    how_old = index.Field(attribute='age')

class Mammoth(grok.Model):
    grok.implements(IMammoth)

    def __init__(self, name, age):
        self.name = name
        self.age = age



from setuptools import setup, find_packages

setup(
    name='grokdocs',
    version='0.1',
    author='Grok Team',
    author_email='grok-dev@zope.org',
    url='http://svn.zope.org/grok/trunk',
    description="""\
Grokdocs: Build the Grok documentation in different formats!
""",
    install_requires=['docutils==0.4',
                      'Sphinx==0.4',
                      'Pygments==0.8.1',
                      ],
    package_dir = {'': 'src'},
    packages=find_packages('src'),
    entry_points="""
    [console_scripts]
    grokdocs2html = grokdocs.grokdocs:render
    grokdocs2latex = grokdocs.grokdocs:render_latex
    """
    )




##############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok interfaces
"""
from zope import interface
from zope.interface.interfaces import IInterface
from zope.publisher.interfaces.http import IHTTPRequest
from zope.app.container.interfaces import IContainer as IContainerBase

# Expose interfaces from grokcore.* packages as well:
import grokcore.component.interfaces
import grokcore.formlib.interfaces
import grokcore.security.interfaces
import grokcore.view.interfaces
import grokcore.viewlet.interfaces

from grokcore.component.interfaces import IContext
from grokcore.component.interfaces import IGrokErrors


class IGrokBaseClasses(grokcore.component.interfaces.IBaseClasses,
                       grokcore.security.interfaces.IBaseClasses,
                       grokcore.view.interfaces.IBaseClasses):
    Model = interface.Attribute("Base class for persistent content objects "
                                "(models).")
    Container = interface.Attribute("Base class for containers.")
    OrderedContainer = interface.Attribute("Base class for ordered containers.")
    Site = interface.Attribute("Mixin class for sites.")
    Application = interface.Attribute("Base class for applications.")
    Annotation = interface.Attribute("Base class for persistent annotations.")
    LocalUtility = interface.Attribute("Base class for local utilities.")
    XMLRPC = interface.Attribute("Base class for XML-RPC methods.")
    JSON = interface.Attribute("Base class for JSON methods.")
    REST = interface.Attribute("Base class for REST views.")
    Traverser = interface.Attribute("Base class for custom traversers.")
    Indexes = interface.Attribute("Base class for catalog index definitions.")
    Role = interface.Attribute("Base class for roles.")


class IGrokDirectives(grokcore.component.interfaces.IDirectives,
                      grokcore.security.interfaces.IDirectives,
                      grokcore.view.interfaces.IDirectives):

    def local_utility(factory, provides=None, name=u'',
                      setup=None, public=False, name_in_container=None):
        """Register a local utility.

        factory - the factory that creates the local utility
        provides - the interface the utility should be looked up with
        name - the name of the utility
        setup - a callable that receives the utility as its single argument,
                it is called after the utility has been created and stored
        public - if False, the utility will be stored below ++etc++site
                 if True, the utility will be stored directly in the site.
                 The site should in this case be a container.
        name_in_container - the name to use for storing the utility
        """

    def permissions(permissions):
        """Specify the permissions that comprise a role.
        """

    def site(class_or_interface):
        """Specifies the site that an indexes definition is for.

        It can only be used inside grok.Indexes subclasses.
        """



class IGrokEvents(interface.Interface):

    IObjectCreatedEvent = interface.Attribute("")

    ObjectCreatedEvent = interface.Attribute("")

    IObjectModifiedEvent = interface.Attribute("")

    ObjectModifiedEvent = interface.Attribute("")

    IObjectCopiedEvent = interface.Attribute("")

    ObjectCopiedEvent = interface.Attribute("")

    IObjectAddedEvent = interface.Attribute("")

    ObjectAddedEvent = interface.Attribute("")

    IObjectMovedEvent = interface.Attribute("")

    ObjectMovedEvent = interface.Attribute("")

    IObjectRemovedEvent = interface.Attribute("")

    ObjectRemovedEvent = interface.Attribute("")

    IContainerModifiedEvent = interface.Attribute("")

    ContainerModifiedEvent = interface.Attribute("")

    IBeforeTraverseEvent = interface.Attribute("")

class IGrokAPI(grokcore.security.interfaces.IGrokcoreSecurityAPI,
               grokcore.view.interfaces.IGrokcoreViewAPI,
               grokcore.viewlet.interfaces.IGrokcoreViewletAPI,
               grokcore.formlib.interfaces.IGrokcoreFormlibAPI,
               IGrokBaseClasses, IGrokDirectives,
               IGrokEvents, IGrokErrors):

    # BBB this is deprecated
    def grok(dotted_name):
        """Grok a module or package specified by ``dotted_name``.

        NOTE: This function will be removed from the public Grok
        public API.  For tests and interpreter sessions, use
        grok.testing.grok().
        """

    # BBB this is deprecated
    def grok_component(name, component, context=None, module_info=None,
                       templates=None):
        """Grok an arbitrary object. Can be useful during testing.

        name - the name of the component (class name, or global instance name
               as it would appear in a module).
        component - the object (class, etc) to grok.
        context - the context object (optional).
        module_info - the module being grokked (optional).
        templates - the templates registry (optional).

        Note that context, module_info and templates might be required
        for some grokkers which rely on them.

        NOTE: This function will be removed from the public Grok
        public API.  For tests and interpreter sessions, use
        grok.testing.grok_component().
        """

    def notify(event):
        """Send ``event`` to event subscribers."""

    def getSite():
        """Get the current site."""


    IRESTSkinType = interface.Attribute('The REST skin type')


class IGrokView(grokcore.view.interfaces.IGrokView):
    """Grok views all provide this interface."""

    def application_url(name=None):
        """Return the URL of the closest application object in the
        hierarchy or the URL of a named object (``name`` parameter)
        relative to the closest application object.
        """

    def flash(message, type='message'):
        """Send a short message to the user."""


class IGrokForm(grokcore.formlib.interfaces.IGrokForm, IGrokView):
    """All Grok forms provides this interface."""


class IREST(interface.Interface):
    context = interface.Attribute("Object that the REST handler presents.")

    request = interface.Attribute("Request that REST handler was looked"
                                  "up with.")

    body = interface.Attribute(
        """The text of the request body.""")

class IApplication(interface.Interface):
    """Marker-interface for grok application factories.

    Used to register applications as utilities to look them up and
    provide a list of grokked applications.
    """

class IIndexDefinition(interface.Interface):
    """Define an index for grok.Indexes.
    """

    def setup(catalog, name, context):
        """Set up index called name in given catalog.

        Use name for index name and attribute to index. Set up
        index for interface or class context.
        """

class IRESTLayer(IHTTPRequest):
    """REST-specific Request functionality.

    Base Interfaces for defining REST-layers.
    """

class IRESTSkinType(IInterface):
    """Skin type for REST requests.
    """

class IContainer(IContext, IContainerBase):
    """A Grok container.
    """

class IGrokSecurityView(interface.Interface):
    """A view treated special by the Grok publisher.

    Views that provide this interface are treated more generously by
    the Grok publisher, as they are allowed to use attributes, which
    are not covered by permission setttings.

    `grok.Permission` and `grok.require` settings however, will be
    applied to such views.
    """



"""
Grok error because import model doesn't count as context:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: No module-level context for
  <class 'grok.tests.adapter.importedmodel2.Painting'>,
  please use the 'context' directive.

"""
import grok
from grok.tests.adapter.adapter import Cave
from zope import interface

class IPainting(interface.Interface):
    pass

class Painting(grok.Adapter):
    """
    Grokking of this should fail because there's no model (only an
    imported one which doesn't count).
    """
    grok.implements(IPainting)



"""
  >>> import grok.tests.utility.local_implementsnone2_fixture
  Traceback (most recent call last):
    ...
  GrokImportError: ("Cannot determine which interface to use for utility
  registration of
  <class 'grok.tests.utility.local_implementsnone2_fixture.Fireplace'>.
  It implements an interface that is a specialization of an interface
  implemented by grok.LocalUtility. Specify the interface by either
  using grok.provides on the utility or passing 'provides' to
  grok.local_utility.",
  <class 'grok.tests.utility.local_implementsnone2_fixture.Fireplace'>)

"""



import grok

grok.definepermission('grok.Complex')  # define permission first

class Complex(grok.Model):
    # this sets the default for all methods
    grok.require('zope.Public')  # this is actually the default

    # this sets the default for reading all attributes that are not methods
    grok.readable('zope.Public')  # this is actually the default

    # this sets the default for writing all attributes that are not methods
    grok.writable('zope.Public')  # this is actually the default

    # this overrides the above
    grok.readable('grok.Complex', 'attr1') # override default
    grok.readable('zope.ManageServices', 'attr2') # override default
    grok.writable('zope.ManageContent', 'attr1', 'attr2') # override default

    def __init__(self):
        self.attr1 = 1
        self.attr2 = 2

    @grok.require('some.permission')
    def doSomethingVerySecret(self):
        pass

    @grok.require('zope.Public')  # this is actually the default
    def imPublic(self):
        pass

class SubClass(Complex):
    # it's all inherited

    grok.readable('zope.Public', 'attr1')




"""
Tests with real TestCase objects.

:Test-Layer: python

"""

import unittest

class TestTest(unittest.TestCase):

    def setUp(self):
        pass

    def testFoo(self):
        self.assertEqual(2, 1+1)




"""
Grok allows you to set up catalog indexes in your application with a
special indexes declaration. We can specify the catalog name using
grok.name.

Let's set up a site in which we manage a couple of objects::

  >>> herd = Herd()
  >>> getRootFolder()['herd'] = herd
  >>> from zope.app.component.hooks import setSite
  >>> setSite(herd)

  >>> from zope.app.catalog.interfaces import ICatalog
  >>> from zope.component import getUtility

We have to look up the catalog by name now::

  >>> catalog = getUtility(ICatalog, 'foo_catalog')
  >>> catalog
  <zope.app.catalog.catalog.Catalog object at ...>

Nuke the catalog and intids in the end, so as not to confuse
other tests::

  >>> sm = herd.getSiteManager()
  >>> from zope.app.catalog.interfaces import ICatalog
  >>> sm.unregisterUtility(catalog, provided=ICatalog, name='foo_catalog')
  True
  >>> from zope.app.intid.interfaces import IIntIds
  >>> from zope import component
  >>> intids = component.getUtility(IIntIds)
  >>> sm.unregisterUtility(intids, provided=IIntIds)
  True

Unfortunately ftests don't have good isolation from each other yet.
"""
import grok
from grok import index

class Herd(grok.Container, grok.Application):
    pass

class Mammoth(grok.Model):
    def __init__(self, name, age, message):
        self.name = name
        self.age = age
        self._message = message

    def message(self):
        return self._message

class MammothIndexes(grok.Indexes):
    grok.context(Mammoth)
    grok.name('foo_catalog')
    grok.site(Herd)
    
    name = index.Field()
    age = index.Field()
    message = index.Text()



#############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grokkers for Grok-configured components.

This `meta` module contains the actual grokker mechanisms for which the
Grok web framework is named.  A directive in the adjacent `meta.zcml`
file directs the `martian` library to scan this file, where it discovers
and registers the grokkers you see below.  The grokkers are then active
and available as `martian` recursively examines the packages and modules
of a Grok-based web application.

"""
import zope.component.interface
from zope import interface, component
from zope.interface.interface import InterfaceClass
from zope.publisher.interfaces.browser import (IDefaultBrowserLayer,
                                               IBrowserPublisher)
from zope.publisher.interfaces.http import IHTTPRequest

from zope.publisher.interfaces.xmlrpc import IXMLRPCRequest
from zope.securitypolicy.interfaces import IRole
from zope.securitypolicy.rolepermission import rolePermissionManager

from zope.annotation.interfaces import IAnnotations

from zope.app.publisher.xmlrpc import MethodPublisher
from zope.app.container.interfaces import IContainer
from zope.app.container.interfaces import INameChooser
from zope.app.container.contained import contained

from zope.app.intid import IntIds
from zope.app.intid.interfaces import IIntIds
from zope.app.catalog.catalog import Catalog
from zope.app.catalog.interfaces import ICatalog

from zope.exceptions.interfaces import DuplicationError

import martian
from martian.error import GrokError
from martian import util

import grok
from grok import components
from grok.util import make_checker
from grok.interfaces import IRESTSkinType

from grokcore.security.meta import PermissionGrokker

from grokcore.view.meta.views import default_fallback_to_name


def default_annotation_provides(factory, module, **data):
    base_interfaces = interface.implementedBy(grok.Annotation)
    factory_interfaces = interface.implementedBy(factory)
    real_interfaces = list(factory_interfaces - base_interfaces)
    util.check_implements_one_from_list(real_interfaces, factory)
    return real_interfaces[0]

def default_annotation_name(factory, module, **data):
    return factory.__module__ + '.' + factory.__name__


class XMLRPCGrokker(martian.MethodGrokker):
    """Grokker for methods of a `grok.XMLRPC` subclass.

    When an application defines a `grok.XMLRPC` view, we do not actually
    register the view with the Component Architecture.  Instead, we grok
    each of its methods separately, placing them each inside of a new
    class that we create on-the-fly by calling `type()`.  We make each
    method the `__call__()` method of its new class, since that is how
    Zope always invokes views.  And it is this new class that is then
    made the object of the two configuration actions that we schedule:
    one to activate it as an XML-RPC adapter for the context, and the
    other to prepare a security check for the adapter.

    """
    martian.component(grok.XMLRPC)
    martian.directive(grok.context)
    martian.directive(grok.require, name='permission')

    def execute(self, factory, method, config, context, permission, **kw):
        name = method.__name__

        # Make sure that the class inherits MethodPublisher, so that the
        # views have a location
        method_view = type(
            factory.__name__, (factory, MethodPublisher),
            {'__call__': method}
            )

        adapts = (context, IXMLRPCRequest)
        config.action(
            discriminator=('adapter', adapts, interface.Interface, name),
            callable=component.provideAdapter,
            args=(method_view, adapts, interface.Interface, name),
            )
        config.action(
            discriminator=('protectName', method_view, '__call__'),
            callable=make_checker,
            args=(factory, method_view, permission),
            )
        return True


class RESTGrokker(martian.MethodGrokker):
    """Grokker for methods of a `grok.REST` subclass.

    When an application defines a `grok.REST` view, we do not actually
    register the view with the Component Architecture.  Instead, we grok
    each of its methods separately, placing them each inside of a new
    class that we create on-the-fly by calling `type()`.  We make each
    method the `__call__()` method of its new class, since that is how
    Zope always invokes views.  And it is this new class that is then
    made the object of the two configuration actions that we schedule:
    one to activate it as a REST adapter for the context, and the other
    to prepare a security check for the adapter.

    This results in several registered views, typically with names like
    `GET`, `PUT`, and `POST` - one for each method that the `grok.REST`
    subclass defines.

    """
    martian.component(grok.REST)
    martian.directive(grok.context)
    martian.directive(grok.layer, default=grok.IRESTLayer)
    martian.directive(grok.require, name='permission')

    def execute(self, factory, method, config, permission, context,
                layer, **kw):
        name = method.__name__

        method_view = type(
            factory.__name__, (factory,),
            {'__call__': method }
            )

        adapts = (context, layer)
        config.action(
            discriminator=('adapter', adapts, interface.Interface, name),
            callable=component.provideAdapter,
            args=(method_view, adapts, interface.Interface, name),
            )
        config.action(
            discriminator=('protectName', method_view, '__call__'),
            callable=make_checker,
            args=(factory, method_view, permission),
            )
        return True


_restskin_not_used = object()

class RestskinInterfaceDirectiveGrokker(martian.InstanceGrokker):
    """Grokker for interfaces providing the `grok.restskin()` directive.

    Applications create REST skins by subclassing `grok.IRESTLayer`
    and providing the subclass with a `grok.restskin()` directive giving
    the prefix string which distinguishes that REST layers from others.
    This grokker registers those skins.

    """
    martian.component(InterfaceClass)

    def grok(self, name, interface, module_info, config, **kw):
        # This `InstanceGrokker` will be called for every instance of
        # `InterfaceClass` - that is, for every interface defined in an
        # application module!  So we have to do our own filtering, by
        # checking whether each interface includes the `grok.restskin()`
        # directive, and skipping those that do not.
        restskin = grok.restskin.bind(default=_restskin_not_used
                                      ).get(interface)
        if restskin is _restskin_not_used:
            # The restskin directive is not actually used on the found
            # interface.
            return False

        if not interface.extends(grok.IRESTLayer):
            # For REST layers it is required to extend IRESTLayer.
            raise GrokError(
                "The grok.restskin() directive is used on interface %r. "
                "However, %r does not extend IRESTLayer which is "
                "required for interfaces that are used as layers and are to "
                "be registered as a restskin."
                % (interface.__identifier__, interface.__identifier__),
                interface
                )
        config.action(
            discriminator=('restprotocol', restskin),
            callable=zope.component.interface.provideInterface,
            args=(restskin, interface, IRESTSkinType)
            )
        return True


class JSONGrokker(martian.MethodGrokker):
    """Grokker for methods of a `grok.JSON` subclass.

    When an application defines a `grok.JSON` view, we do not actually
    register the view with the Component Architecture.  Instead, we grok
    each of its methods separately, placing them each inside of a new
    class that we create on-the-fly by calling `type()`.  We make each
    method the `__call__()` method of its new class, since that is how
    Zope always invokes views.  And it is this new class that is then
    made the object of the two configuration actions that we schedule:
    one to activate it as a JSON adapter for the context, and the other
    to prepare a security check for the adapter.

    """
    martian.component(grok.JSON)
    martian.directive(grok.context)
    martian.directive(grok.require, name='permission')
    martian.directive(grok.layer, default=IDefaultBrowserLayer)

    def execute(
            self, factory, method, config, context, permission, layer, **kw):
        # Create a new class with a __view_name__ attribute so the
        # JSON class knows what method to call.
        method_view = type(
            factory.__name__, (factory,),
            {'__view_name__': method.__name__}
            )
        adapts = (context, layer)
        name = method.__name__
        config.action(
            discriminator=('adapter', adapts, interface.Interface, name),
            callable=component.provideAdapter,
            args=(method_view, adapts, interface.Interface, name),
            )
        config.action(
            discriminator=('protectName', method_view, '__call__'),
            callable=make_checker,
            args=(factory, method_view, permission),
            )
        return True


class TraverserGrokker(martian.ClassGrokker):
    """Grokker for subclasses of `grok.Traverser`."""
    martian.component(grok.Traverser)
    martian.directive(grok.context)

    def execute(self, factory, config, context, **kw):
        adapts = (context, IHTTPRequest)
        config.action(
            discriminator=('adapter', adapts, IBrowserPublisher, ''),
            callable=component.provideAdapter,
            args=(factory, adapts, IBrowserPublisher),
            )
        return True


class SiteGrokker(martian.ClassGrokker):
    """Grokker for subclasses of `grok.Site`."""
    martian.component(grok.Site)
    martian.priority(500)
    martian.directive(grok.local_utility, name='infos')

    def execute(self, factory, config, infos, **kw):
        if not infos:
            return False

        infos = infos.values()
        for info in infos:
            if info.public and not IContainer.implementedBy(factory):
                raise GrokError(
                    "Cannot set public to True with grok.local_utility as "
                    "the site (%r) is not a container." %
                    factory, factory)

        # Store the list of info objects in their "natural" order on the
        # site class. They will be picked up by a subscriber doing the
        # actual registrations in definition order.
        factory.__grok_utilities_to_install__ = sorted(infos)
        adapts = (factory, grok.IObjectAddedEvent)

        config.action(
            discriminator=None,
            callable=component.provideHandler,
            args=(localUtilityRegistrationSubscriber, adapts),
            )
        return True


def localUtilityRegistrationSubscriber(site, event):
    """A subscriber that fires to set up local utilities.
    """
    installed = getattr(site, '__grok_utilities_installed__', False)
    if installed:
        return

    for info in getattr(site.__class__, '__grok_utilities_to_install__', []):
        setupUtility(site, info.factory(), info.provides, name=info.name,
                     name_in_container=info.name_in_container,
                     public=info.public, setup=info.setup)

    # we are done. If this subscriber gets fired again, we therefore
    # do not register utilities anymore
    site.__grok_utilities_installed__ = True


def setupUtility(site, utility, provides, name=u'',
                 name_in_container=None, public=False, setup=None):
    """Set up a utility in a site.

    site - the site to set up the utility in
    utility - the utility to set up
    provides - the interface the utility should be registered with
    name - the name the utility should be registered under, default
      the empty string (no name)
    name_in_container - if given it will be used to add the utility
      object to its container. Otherwise a name will be made up
    public - if False, the utility will be stored in the site manager. If
      True, the utility will be storedin the site (it is assumed the
      site is a container)
    setup - if not None, it will be called with the utility as its first
       argument. This function can then be used to further set up the
       utility.
    """
    site_manager = site.getSiteManager()

    if not public:
        container = site_manager
    else:
        container = site

    if name_in_container is None:
        name_in_container = INameChooser(container).chooseName(
            utility.__class__.__name__, utility)
    container[name_in_container] = utility

    if setup is not None:
        setup(utility)

    site_manager.registerUtility(utility, provided=provides,
                                 name=name)


class RoleGrokker(martian.ClassGrokker):
    """Grokker for components subclassed from `grok.Role`.

    Each role is registered as a global utility providing the service
    `IRole` under its own particular name, and then granted every
    permission named in its `grok.permission()` directive.

    """
    martian.component(grok.Role)
    martian.priority(martian.priority.bind().get(PermissionGrokker()) - 1)
    martian.directive(grok.name)
    martian.directive(grok.title, get_default=default_fallback_to_name)
    martian.directive(grok.description)
    martian.directive(grok.permissions)

    def execute(self, factory, config, name, title, description,
                permissions, **kw):
        if not name:
            raise GrokError(
                "A role needs to have a dotted name for its id. Use "
                "grok.name to specify one.", factory)
        # We can safely convert to unicode, since the directives makes sure
        # it is either unicode already or ASCII.
        role = factory(unicode(name), unicode(title), unicode(description))

        config.action(
            discriminator=('utility', IRole, name),
            callable=component.provideUtility,
            args=(role, IRole, name),
            )

        for permission in permissions:
            config.action(
                discriminator=('grantPermissionToRole', permission, name),
                callable=rolePermissionManager.grantPermissionToRole,
                args=(permission, name),
                )
        return True


class AnnotationGrokker(martian.ClassGrokker):
    """Grokker for components subclassed from `grok.Annotation`."""
    martian.component(grok.Annotation)
    martian.directive(grok.context, name='adapter_context')
    martian.directive(grok.provides, get_default=default_annotation_provides)
    martian.directive(grok.name, get_default=default_annotation_name)

    def execute(self, factory, config, adapter_context, provides, name, **kw):
        @component.adapter(adapter_context)
        @interface.implementer(provides)
        def getAnnotation(context):
            annotations = IAnnotations(context)
            try:
                result = annotations[name]
            except KeyError:
                result = factory()
                annotations[name] = result

            # Containment has to be set up late to allow containment
            # proxies to be applied, if needed. This does not trigger
            # an event and is idempotent if containment is set up
            # already.
            contained_result = contained(result, context, name)
            return contained_result

        config.action(
            discriminator=('adapter', adapter_context, provides, ''),
            callable=component.provideAdapter,
            args=(getAnnotation,),
            )
        return True


class ApplicationGrokker(martian.ClassGrokker):
    """Grokker for Grok application classes."""
    martian.component(grok.Application)
    martian.priority(500)

    def grok(self, name, factory, module_info, config, **kw):
        # XXX fail loudly if the same application name is used twice.
        provides = grok.interfaces.IApplication
        name = '%s.%s' % (module_info.dotted_name, name)
        config.action(
            discriminator=('utility', provides, name),
            callable=component.provideUtility,
            args=(factory, provides, name),
            )
        return True


class IndexesGrokker(martian.InstanceGrokker):
    """Grokker for Grok index bundles."""
    martian.component(components.IndexesClass)

    def grok(self, name, factory, module_info, config, **kw):
        site = grok.site.bind().get(factory)
        context = grok.context.bind().get(factory, module_info.getModule())
        catalog_name = grok.name.bind().get(factory)

        if site is None:
            raise GrokError("No site specified for grok.Indexes "
                            "subclass in module %r. "
                            "Use grok.site() to specify."
                            % module_info.getModule(),
                            factory)
        indexes = getattr(factory, '__grok_indexes__', None)
        if indexes is None:
            return False

        subscriber = IndexesSetupSubscriber(catalog_name, indexes,
                                            context, module_info)
        subscribed = (site, grok.IObjectAddedEvent)
        config.action(
            discriminator=None,
            callable=component.provideHandler,
            args=(subscriber, subscribed),
            )
        return True


class IndexesSetupSubscriber(object):
    """Helper that sets up indexes when their Grok site is created.

    Each `grok.Indexes` class serves as an assertion that, whenever an
    instance of its `grok.site()` is created, the given list of indexes
    should be generated as well.  But a long period of time could elapse
    between when the application starts (and its indexes are grokked),
    and the moment, maybe days or weeks later, when a new instance of
    that `grok.Site` is created.  Hence this `IndexesSetupSubscriber`:
    it can be instantiated at grokking time with the index information,
    and then registered with the Component Architecture as an event that
    should be fired later, whenever the right kind of `grok.Site` is
    instantiated.  At that point its `__call__` method is kicked off and
    it makes sure the index catalogs get created properly.

    """
    def __init__(self, catalog_name, indexes, context, module_info):
        self.catalog_name = catalog_name
        self.indexes = indexes
        self.context = context
        self.module_info = module_info

    def __call__(self, site, event):
        # make sure we have an intids
        self._createIntIds(site)
        # get the catalog
        catalog = self._createCatalog(site)
        # now install indexes
        for name, index in self.indexes.items():
            try:
                index.setup(catalog, name, self.context, self.module_info)
            except DuplicationError:
                raise GrokError(
                    "grok.Indexes in module %r causes "
                    "creation of catalog index %r in catalog %r, "
                    "but an index with that name is already present." %
                    (self.module_info.getModule(), name, self.catalog_name),
                    None)

    def _createCatalog(self, site):
        """Create the catalog if needed and return it.

        If the catalog already exists, return that.

        """
        catalog = zope.component.queryUtility(
            ICatalog, name=self.catalog_name, context=site, default=None)
        if catalog is not None:
            return catalog
        catalog = Catalog()
        setupUtility(site, catalog, ICatalog, name=self.catalog_name)
        return catalog

    def _createIntIds(self, site):
        """Create intids if needed, and return it.
        """
        intids = zope.component.queryUtility(
            IIntIds, context=site, default=None)
        if intids is not None:
            return intids
        intids = IntIds()
        setupUtility(site, intids, IIntIds)
        return intids




"""
Anything can be registered as a local utility. If it implements a single
interface, there is no need to specify which interface it provides.

In this test, the utility does not implement any interface, so it cannot be
registered as a local utility.

  >>> import grok.tests.utility.local_implementsnone_fixture
  Traceback (most recent call last):
    ...
  GrokError: <class 'grok.tests.utility.local_implementsnone_fixture.Fireplace'>
  must implement at least one interface (use grok.implements to specify).

"""



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Builds HTML and Latex versions of the Official Grok Documentation using Sphinx.
"""
import sys
import os.path
from shutil import copyfile
import getopt
from docutils import nodes
from docutils.parsers.rst import directives
from pygments import highlight
from pygments.lexers import get_lexer_by_name, TextLexer
from pygments.formatters import LatexFormatter
import sphinx
from sphinx.util.console import nocolor
import latex_hacks

ROOT = os.path.join(os.path.dirname(
    os.path.abspath(__file__)), '..', '..', '..') 
SRCDIR_ALL = os.path.join(ROOT, 'doc')
SRCDIR_REF = os.path.join(SRCDIR_ALL, 'reference')

BUILD_DIR = os.path.join(ROOT, 'build')

HTMLDIR_ALL = os.path.join(BUILD_DIR, 'html')
HTMLDIR_REF = os.path.join(BUILD_DIR, 'html', 'reference')

LATEX_ALL = os.path.join(BUILD_DIR, 'latex')

LATEX_SETTINGS = {
    'DEFAULT': LatexFormatter(),
    'VARIANTS' : {
       'linenos' : LatexFormatter(linenos=True),
       'nolinenos' : LatexFormatter(linenos=False)
       },
    'FORMAT': 'latex',
    }

for build_path in (BUILD_DIR, HTMLDIR_ALL, HTMLDIR_REF, LATEX_ALL):
    # Create the output directories, if they do not exist...
    if not os.path.exists(build_path):
        os.makedirs(build_path)


def pygments_latex_directive(name, arguments, options, content, lineno,
                             content_offset, block_text, state, state_machine):
    """A docutils directive that provides syntax highlighting for LaTeX.

    This is needed to circumvent highlighting quirks when doing
    non-HTML output. The pygments_directive delivers plain HTML, which
    we must avoid when generating LaTeX for example.
    """
    try:
        lexer = get_lexer_by_name(arguments[0])
    except ValueError:
        # no lexer found - use the text one instead of an exception
        lexer = TextLexer()
    # take an arbitrary option if more than one is given
    formatter = options and LATEX_SETTINGS['VARIANTS'][
        options.keys()[0]] or LATEX_SETTINGS['DEFAULT']
    parsed = highlight(u'\n'.join(content), lexer, formatter)
    return [nodes.raw('', parsed, format=LATEX_SETTINGS['FORMAT'])]

pygments_latex_directive.arguments = (1, 0, 1)
pygments_latex_directive.content = 1



def usage(argv, msg=None, default_src=None, default_out=None):
    """Some hints for users.

    Adapted from sphinx __init__. Because we add an `-h` option and
    provide a slightliy different syntax than stock sphinx (srcdir and
    targetdir have defaults here), we also need our own help texts.
    """

    if msg:
        print >>sys.stderr, msg
        print >>sys.stderr
    print >>sys.stderr, """\
usage: %s [options] [sourcedir [outdir [filenames...]]]
options: -b <builder> -- builder to use; default is html
         -a        -- write all files; default is to only write new and changed files
         -E        -- don't use a saved environment, always read all files
         -d <path> -- path for the cached environment and doctree files
                      (default outdir/.doctrees)
         -D <setting=value> -- override a setting in sourcedir/conf.py
         -N        -- do not do colored output
         -q        -- no output on stdout, just warnings on stderr
         -P        -- run Pdb on exception
         -h        -- print this help

default sourcedir is %s
default outputdir is %s

modi:
* without -a and without filenames, write new and changed files.
* with -a, write all files.
* with filenames, write these.""" % (argv[0],default_src, default_out)

def usage_grokdoc(argv, msg=None):
    """Wrapper that displays source and target of all docs.
    """
    return usage(argv, msg=msg, default_src=SRCDIR_ALL,
                 default_out=HTMLDIR_ALL)

def usage_grokref(argv, msg=None):
    """Wrapper that displays source and target of reference docs.
    """
    return usage(argv, msg=msg, default_src=SRCDIR_REF,
                 default_out=HTMLDIR_REF)


def render(argv=sys.argv, srcdir=SRCDIR_ALL, htmldir=HTMLDIR_ALL,
             latexdir=LATEX_ALL, do_latex=False):
    """Generate the whole docs, including howtos, reference, etc.
    """
    if srcdir == SRCDIR_ALL:
        sphinx.usage = usage_grokdoc
    if not sys.stdout.isatty() or sys.platform == 'win32':
        # Windows' poor cmd box doesn't understand ANSI sequences
        nocolor()
    opts, args = None, None
    try:
        opts, args = getopt.getopt(argv[1:], 'ab:d:D:NEqPh')
    except getopt.error:
        # sphinx will handle that errors
        pass
    if len(args) < 1:
        argv.append(srcdir)
    if len(args) < 2:
        argv.append(htmldir)

    if opts and '-h' in [x for x,y in opts]:
        sphinx.usage(argv, msg=None)
        return 1

    format = 'html'
    if do_latex:
        format = 'latex'
    if opts and '-b' in [x for x,y in opts]:
        val = filter(lambda x: x[0] == '-b', opts)
        format = val[-1][1]

    if format == 'latex':
        # disable code-block directive by substituting it with a
        # LaTeX-specialized version...
        directives.register_directive('sourcecode', pygments_latex_directive)
        directives.register_directive('code-block', pygments_latex_directive)

        # Set default sourcedir...
        if len(args) < 2:
            argv[-1] = latexdir
        # Copy fncychap.sty to targetdir...
        if os.path.isdir(argv[-1]):
            copyfile(os.path.join(os.path.dirname(__file__),
                                  'texinputs', 'fncychap.sty'),
                     os.path.join(argv[-1], 'fncychap.sty'))
        # Let sphinx know, we want LaTeX...
        argv.insert(1, ('latex'))
        argv.insert(1, ('-b'))
            
    args = argv                 

    print "Source directory is: ", argv[-2]
    print "Target directory is: ", argv[-1]
    print "(run `%s -h` to see the options available)" % argv[0]

    sphinx.main(argv)

    print "Generated docs are in %s." % os.path.abspath(argv[-1])

def render_latex(argv=sys.argv):
    """Generate all docs in LaTeX.
    """
    return render(argv, do_latex=True)

def grokref(argv=sys.argv, do_latex=False):
    """Generate the reference docs.
    """
    sphinx.usage = usage_grokref
    return render(argv, srcdir=SRCDIR_REF, htmldir=HTMLDIR_REF,
                    latexdir=LATEX_ALL, do_latex=do_latex)

def grokref_latex(argv=sys.argv):
    """Generate reference in LaTeX format.
    """
    return render(argv, do_latex=True)




"""
  >>> grok.testing.grok(__name__)
  >>> from zope import component
  >>> from zope.annotation.attribute import AttributeAnnotations
  >>> component.provideAdapter(AttributeAnnotations)

We can adapt a model to an annotation interface and obtain a
persistent annotation storage:

  >>> manfred = Mammoth()
  >>> branding = IBranding(manfred)
  >>> branding.addBrand('mine')
  >>> branding.addBrand('yours')

Regetting the adapter will yield the same annotation storage:

  >>> brands = IBranding(manfred).getBrands()
  >>> brands.sort()
  >>> brands
  ['mine', 'yours']

"""

import grok
from zope import interface
from BTrees.OOBTree import OOTreeSet

class Mammoth(grok.Model):
    pass

class IBranding(interface.Interface):

    def addBrand(brand):
        """Brand an animal with ``brand``, a string."""

    def getBrands():
        """Return a list of brands."""

class Branding(grok.Annotation):
    grok.implements(IBranding)

    def __init__(self): 
        self._brands = OOTreeSet()

    def addBrand(self, brand):
        self._brands.insert(brand)

    def getBrands(self):
        return list(self._brands)



"""

The handleException() method has a special case that might introduce a proxy
and cause the grok security to fail, we have a simple test here that assures
that we don't hit this:

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.open("http://localhost/@@cave")
  Traceback (most recent call last):
  HTTPError: HTTP Error 500: Internal Server Error
  >>> browser.contents
  "It's gone!"

"""

import zope.interface

import grok


class CaveWasRobbedError(Exception):
    pass


class Cave(grok.View):

    grok.context(zope.interface.Interface)

    fire = "It's gone!"

    def render(self):
        raise CaveWasRobbedError("EVERYTHING GONE! GROK ANGRY!")


class CaveErrorView(grok.View):

    grok.context(CaveWasRobbedError)
    grok.name("index.html")

    def render(self):
        self.request.response.setStatus(500)
        return self.context.__parent__.fire



from setuptools import setup, find_packages

version = 0.0

setup(name='Sample',
      version=version,
      description="",
      long_description="""\
""",
      # Get strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[], 
      keywords="",
      author="",
      author_email="",
      url="",
      license="",
      package_dir={'': 'src'},
      packages=find_packages('src'),
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'grok',
                        # Add extra requirements here
                        ],
      entry_points="""
      # Add entry points here
      """,
      )



import os.path
from zope.app.testing.functional import ZCMLLayer
from grok.testing import register_all_tests
samplelayer = ZCMLLayer(
    os.path.join(os.path.dirname(__file__), 'sample.zcml'),
    __name__, 'CustomSampleLayer')
test_suite = register_all_tests(
    'grok.tests.testsetup.cave',
    layer = samplelayer
    )



import grok
  
class Sample(grok.Application, grok.Container):
    pass

class Another(grok.Application, grok.Model):
    pass

class SampleIndex(grok.View):
    grok.context(Sample)
    grok.name('index')
    
class AnotherIndex(grok.View):
    grok.context(Another)
    grok.name('index')



"""
Let's test whether require decorators work for json methods.

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False

We can access the public method just fine::

  >>> browser.open('http://localhost/stomp')
  >>> print browser.contents
  {"Manfred stomped.": ""}

We cannot access the protected method however::

  >>> browser.open('http://localhost/dance')
  Traceback (most recent call last):
    ...
  Unauthorized: (<grok.meta.MammothJSON object at ...>, '__call__', 'zope.ManageContent')

Let's log in as the manager now. We should be able to access the method now::

  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  >>> browser.open('http://localhost/dance')
  >>> print browser.contents
  {"Manfred doesn't like to dance.": ""}
  
"""

import grok
import zope.interface

class MammothJSON(grok.JSON):
    grok.context(zope.interface.Interface)

    def stomp(self):
        return {'Manfred stomped.': ''}

    @grok.require('zope.ManageContent')
    def dance(self):
        return {'Manfred doesn\'t like to dance.': ''}



"""

Let's first grok the meta module to define some basic grokkers::

  >>> import grok
  >>> grok.testing.grok('grok.meta')
  
It is possible to grok an individual component. Let's define an adapter::

  >>> from zope.interface import Interface
  >>> class IMyInterface(Interface):
  ...   pass
  >>> class SomeClass(object):
  ...   pass
  >>> class MyAdapter(grok.Adapter):
  ...   grok.provides(IMyInterface)
  ...   grok.context(SomeClass)

To grok this adapter, you can simply write this::

  >>> grok.testing.grok_component('MyAdapter', MyAdapter)
  True

We can now use the adapter::

  >>> instance = SomeClass()
  >>> adapter = IMyInterface(instance)
  >>> isinstance(adapter, MyAdapter)
  True

We can use grok_component with only two arguments because we know the
adapter grokker is not looking for more. Sometimes we need to supply
an extra argument however::

  >>> class ISecondInterface(Interface):
  ...   pass
  >>> class SecondAdapter(grok.Adapter):
  ...   grok.provides(ISecondInterface)

This adapter does not supply its own context. Trying to do what we did
before will therefore fail::

  >>> grok.testing.grok_component('SecondAdapter', SecondAdapter)
  Traceback (most recent call last):
    ...
  GrokError: No module-level context for <class 'grok.tests.grokker.grokcomponent.SecondAdapter'>, please use the 'context' directive.

So we need to supply the context ourselves::

  >>> grok.testing.grok_component('SecondAdapter', SecondAdapter, context=SomeClass)
  True

Now we can use the SecondAdapter as well::

  >>> adapter = ISecondInterface(instance)
  >>> isinstance(adapter, SecondAdapter)
  True

The next optional argument is module_info and the final argument is
templates.
"""



import grok
from zope import interface
import persistent

class ISpecialPersistent(persistent.interfaces.IPersistent):
    pass

class Fireplace(grok.LocalUtility):
    grok.implements(ISpecialPersistent)

class Cave(grok.Model, grok.Site):
    grok.local_utility(Fireplace)



"""
Models can expose attributes using the grok.traversable directive.

  >>> getRootFolder()["traversefoo"] = Foo('foo')

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False

As always, we can access a model with a view::
  >>> browser.open("http://localhost/traversefoo/")
  >>> print browser.contents
  foo

'foo' is an exposed attribute, so it should be accessible::
  >>> browser.open("http://localhost/traversefoo/foo")
  >>> print browser.contents
  foo

'bar' is an exposed method, and should also be accessible::
  >>> browser.open("http://localhost/traversefoo/bar")
  >>> print browser.contents
  bar

'bar' is also exposed under the name 'namedbar', and can also be accessed::
  >>> browser.open("http://localhost/traversefoo/namedbar")
  >>> print browser.contents
  bar

Finally, attributes which are not exposed, should not be visible:
  >>> browser.open("http://localhost/traversefoo/z")
  Traceback (most recent call last):
  ...
  NotFound: ...

"""
import grok

class Bar(grok.Model):
    def __init__(self, name):
        self.name = name

class BarIndex(grok.View):
    grok.context(Bar)
    grok.name('index')

    def render(self):
        return self.context.name

class Foo(grok.Model):
    grok.traversable('bar')
    grok.traversable('foo')
    grok.traversable(attr='bar', name='namedbar')

    def __init__(self, name):
        self.name = name

    foo = Bar('foo')
    def bar(self):
        return Bar('bar')
    z = "i'm not called"

class FooIndex(grok.View):
    grok.context(Foo)
    grok.name('index')
    def render(self):
        return self.context.name



##############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Base classes for Grok application components.

When an application developer builds a Grok-based application, the
classes they define each typically inherit from one of the base classes
provided here.

"""
import persistent
import simplejson

import zope.location
from zope import component
from zope import interface
from zope.securitypolicy.role import Role as securitypolicy_Role
from zope.publisher.browser import BrowserPage
from zope.publisher.interfaces import NotFound
from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.publisher.interfaces.http import IHTTPRequest
from zope.publisher.publish import mapply
from zope.annotation.interfaces import IAttributeAnnotatable

from zope.app.publisher.browser import getDefaultViewName
from zope.app.container.btree import BTreeContainer
from zope.app.container.contained import Contained
from zope.app.container.interfaces import IReadContainer, IObjectAddedEvent
from zope.app.container.interfaces import IOrderedContainer
from zope.app.container.contained import notifyContainerModified
from persistent.list import PersistentList
from zope.app.component.site import SiteManagerContainer
from zope.app.component.site import LocalSiteManager

import grok
import z3c.flashmessage.interfaces
import martian.util

import grokcore.view
from grok import interfaces, util


class Model(Contained, persistent.Persistent):
    # XXX Inheritance order is important here. If we reverse this,
    # then containers can't be models anymore because no unambigous MRO
    # can be established.
    """The base class for models in Grok applications.

    When an application class inherits from `grok.Model`, it gains the
    ability to persist itself in the Zope object database along with all
    of its attributes, and to remember the container in which it has
    been placed (its "parent") so that its URL can be computed.  It also
    inherits the `IContext` marker interface, which can make it the
    default context for views in its module; the rule is that if a
    module contains `grok.View` classes or other adapters that do not
    define a `grok.context()`, but the module also defines exactly one
    class that provides the `IContext` interface, then that class will
    automatically be made the `grok.context()` of each of the views.

    """
    interface.implements(IAttributeAnnotatable, interfaces.IContext)


class Container(BTreeContainer):
    """The base class for containers in Grok applications.

    When an application class inherits from `grok.Container`, it not
    only has the features of a `grok.Model` (persistance, location, and
    the ability to serve as a default context for other classes), but it
    also behaves like a persistent dictionary.  To store items inside a
    container, simply use the standard Python getitem/setitem protocol::

        mycontainer['counter'] = 72
        mycontainer['address'] = mymodel
        mycontainer['subfolder'] = another_container

    By default, the URL of each item inside a container is the
    container's own URL followed by a slash and the key (like 'counter'
    or 'address') under which that item has been stored.

    """
    interface.implements(IAttributeAnnotatable, interfaces.IContainer)


class OrderedContainer(Container):
    """A Grok container that remembers the order of its items.

    This straightforward extension of the basic `grok.Container`
    remembers the order in which items have been inserted, so that
    `keys()`, `values()`, `items()`, and iteration across the container
    can all return the items in the order they were inserted.  The only
    way of changing the order is to call the `updateOrder()` method.

    """
    interface.implements(IOrderedContainer)

    def __init__(self):
        super(OrderedContainer, self).__init__()
        self._order = PersistentList()

    def keys(self):
        # Return a copy of the list to prevent accidental modifications.
        return self._order[:]

    def __iter__(self):
        return iter(self.keys())

    def values(self):
        return (self[key] for key in self._order)

    def items(self):
        return ((key, self[key]) for key in self._order)

    def __setitem__(self, key, object):
        foo = self.has_key(key)
        # Then do whatever containers normally do.
        super(OrderedContainer, self).__setitem__(key, object)
        if not foo:
            self._order.append(key)

    def __delitem__(self, key):
        # First do whatever containers normally do.
        super(OrderedContainer, self).__delitem__(key)
        self._order.remove(key)

    def updateOrder(self, order):
        """Impose a new order on the items in this container.

        Items in this container are, by default, returned in the order
        in which they were inserted.  To change the order, provide an
        argument to this method that is a sequence containing every key
        already in the container, but in a new order.

        """
        if set(order) != set(self._order):
            raise ValueError("Incompatible key set.")

        self._order = PersistentList()
        self._order.extend(order)
        notifyContainerModified(self)


class Site(SiteManagerContainer):
    """Mixin for creating sites in Grok applications.

    When an application `grok.Model` or `grok.Container` also inherits
    from `grok.Site`, then it can additionally support the registration
    of local Component Architecture entities like `grok.LocalUtility`
    and `grok.Indexes` objects; see those classes for more information.

    """


@component.adapter(Site, IObjectAddedEvent)
def addSiteHandler(site, event):
    """Add a local site manager to a Grok site object upon its creation.

    Grok registers this function so that it gets called each time a
    `grok.Site` instance is added to a container.  It creates a local
    site manager and installs it on the newly created site.

    """
    sitemanager = LocalSiteManager(site)
    # LocalSiteManager creates the 'default' folder in its __init__.
    # It's not needed anymore in new versions of Zope 3, therefore we
    # remove it
    del sitemanager['default']
    site.setSiteManager(sitemanager)


class Application(Site):
    """Mixin for creating Grok application objects.

    When a `grok.Container` (or a `grok.Model`, though most developers
    use containers) also inherits from `grok.Application`, it not only
    gains the component registration abilities of a `grok.Site`, but
    will also be listed in the Grok admin control panel as one of the
    applications that the admin can install directly at the root of
    their Zope database.

    """
    interface.implements(interfaces.IApplication)


class LocalUtility(Model):
    """The base class for local utilities in Grok applications.

    Although application developers can create local utilies without
    actually subclassing `grok.LocalUtility`, they gain three benefits
    from doing so.  First, their code is more readable because their
    classes "look like" local utilities to casual readers.  Second,
    their utility will know how to persist itself to the Zope database,
    which means that they can set its object attributes and know that
    the values are getting automatically saved.  Third, they can omit
    the `grok.provides()` directive naming the interface that the
    utility provides, if their class only `grok.implements()` a single
    interface (unless the interface is one that the `grok.LocalUtility`
    already implements, in which case Grok cannot tell them apart, and
    `grok.provides()` must be used explicitly anyway).

    """


class Annotation(persistent.Persistent):
    """The base class for annotation classes in Grok applications."""


class View(grokcore.view.View):
    """The base class for views in Grok applications.

    Each class that inherits from `grok.View` is designed to "render" a
    category of content objects by reducing them to a document (often an
    HTML document).  Every view has a name, and is invoked when users
    visit the URL of an eligible context object followed by the name of
    the view itself::

        http://example.com/app/folder/object/viewname

    If the view name might conflict with actual content inside of the
    context (in the above URL, the context might already contain an
    attribute or item named ``viewname``), then the URL can be explicit
    that it is asking for a view by preceding its name with ``@@``::

        http://example.com/app/folder/object/@@viewname

    Instead of returning a full document, views are sometimes used to
    provide only a snippet of information for inclusion in some larger
    document; the view can then be called from inside of another view's
    page template::

        <li tal:content="context/@@viewname">snippet goes here</li>

    A view class can specify the category of objects that it can render
    by calling the `grok.context()` directive with either a class or an
    interface.  Otherwise, Grok will attempt to determine the context
    automatically by searching the view's module for exactly one
    `grok.Model` or `grok.Container` class (or some other class
    providing the interface `IContext`) and using that class, if found.

    Grok normally creates a view's name (the name used in URLs) by
    downcasing the name of the view class itself.  The developer can
    override this by supplying the `grok.name()` directive instead.

    The view name ``index`` is special (this works whether the view
    class itself is named ``Index``, or whether ``grok.name('index')``
    is used instead).  A view named ``index`` is used to render an
    object when the user visits its URL without appending a view name.

    Each view needs to generate and return a document. There are two
    ways of doing so: either the view can provide a `render()` method
    that returns a document, or the view can be associated with a page
    template that Grok will.  Page templates can be associated with a
    view in three different ways:

    * Grok will automatically associate a view with a page template
      defined in an accompanying ``templates`` directory.  If a view
      class ``MammothList`` occurs in a module ``<src>/animal.py``, for
      example, then Grok will look for a page template with the name
      ``<src>/animal_templates/mammothlist.pt``, where ``.pt`` can be
      any page-template extension recognized by Grok.

    * Grok will automatically associate a view with a page template
      object in the same module whose name is the downcased name of the
      view class itself.  For example, a view ``MammothList`` might be
      defined in a module alongside an actual template instance named
      ``mammothlist``.

    * The developer can explicitly define the path to the page template
      file by providing the ``grok.template()`` directive.

    Before a page template is rendered, Grok will call the `update()`
    method on the view, if one is supplied, which can pre-compute values
    that the template will need to display.  Both `render()` methods and
    `update()` methods will find the context for which the view is being
    rendered under ``self.context``.

    """
    interface.implements(interfaces.IGrokView)

    def application_url(self, name=None):
        """Return the URL of the nearest enclosing `grok.Application`."""
        obj = self.context
        while obj is not None:
            if isinstance(obj, Application):
                return self.url(obj, name)
            obj = obj.__parent__
        raise ValueError("No application found.")

    def flash(self, message, type='message'):
        """Send a short message to the user."""
        # XXX this has no tests or documentation, anywhere
        source = component.getUtility(
            z3c.flashmessage.interfaces.IMessageSource, name='session')
        source.send(message, type)


class Form(grokcore.formlib.Form, View):
    """The base class for forms in Grok applications.

    A class that inherits from `grok.Form` is a `grok.View` whose
    template will be given information about the fields in its context,
    and use that information to render an HTML form for adding or
    editing the form.  Generally developers use one of the subclasses:

    * `grok.AddForm`
    * `grok.DisplayForm`
    * `grok.EditForm`

    """
    interface.implements(interfaces.IGrokForm)


class AddForm(grokcore.formlib.AddForm, View):
    """Base class for add forms in Grok applications."""
    interface.implements(interfaces.IGrokForm)


class DisplayForm(grokcore.formlib.DisplayForm, View):
    """Base class for display forms in Grok applications."""
    interface.implements(interfaces.IGrokForm)


class EditForm(grokcore.formlib.EditForm, View):
    """Base class for edit forms in Grok applications."""
    interface.implements(interfaces.IGrokForm)


class XMLRPC(object):
    """Base class for XML-RPC endpoints in Grok applications.

    When an application creates a subclass of `grok.XMLRPC`, it is
    creating an XML-RPC view.  Like other Grok views, each `grok.XMLRPC`
    component can either use an explicit `grok.context()` directive to
    specify the kind of object it wraps, or else Grok will look through
    the same module for exactly one `grok.Model` or `grok.Container` (or
    other `IGrokContext` implementor) and make that class its context
    instead.

    Every object that is an instance of the wrapped class or interface
    becomes a legitimate XML-RPC server URL, offering as available
    procedures whatever methods have been defined inside of that
    `grok.XMLRPC` component.  When a method is called over XML-RPC, any
    parameters are translated into normal Python data types and supplied
    as normal positional arguments.  When the method returns a value or
    raises an exception, the result is converted back into an XML-RPC
    response for the client.  In both directions, values are marshalled
    transparently to and from XML-RPC data structures.

    During the execution of an XML-RPC method, the object whose URL was
    used for the XML-RPC call is available as ``self.context``.

    """


class REST(zope.location.Location):
    """Base class for REST views in Grok applications."""
    interface.implements(interfaces.IREST)

    def __init__(self, context, request):
        self.context = self.__parent__ = context
        self.request = request
        self.body = request.bodyStream.getCacheStream().read()

    @property
    def response(self):
        return self.request.response


class JSON(BrowserPage):
    """Base class for JSON views in Grok applications."""
    interface.implements(interfaces.IGrokSecurityView)

    def __call__(self):
        view_name = self.__view_name__
        method = getattr(self, view_name)
        method_result = mapply(method, (), self.request)
        return simplejson.dumps(method_result)


class Traverser(object):
    """Base class for traversers in Grok applications."""
    interface.implements(IBrowserPublisher)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def browserDefault(self, request):
        # if we have a RESTful request, we will handle
        # GET, POST and HEAD differently (PUT and DELETE are handled already
        # but not on the BrowserRequest layer but the HTTPRequest layer)
        if interfaces.IRESTLayer.providedBy(request):
            rest_view = component.getMultiAdapter(
                (self.context, self.request), name=request.method)
            return rest_view, ()
        view_name = getDefaultViewName(self.context, request)
        view_uri = "@@%s" % view_name
        return self.context, (view_uri,)

    def publishTraverse(self, request, name):
        subob = self.traverse(name)
        if subob is not None:
            return util.safely_locate_maybe(subob, self.context, name)

        traversable_dict = grok.traversable.bind().get(self.context)
        if traversable_dict:
            if name in traversable_dict:
                subob = getattr(self.context, traversable_dict[name])
                if callable(subob):
                    subob = subob()
                return util.safely_locate_maybe(subob, self.context, name)

        # XXX Special logic here to deal with containers.  It would be
        # good if we wouldn't have to do this here. One solution is to
        # rip this out and make you subclass ContainerTraverser if you
        # wanted to override the traversal behaviour of containers.
        if IReadContainer.providedBy(self.context):
            item = self.context.get(name)
            if item is not None:
                return item

        view = component.queryMultiAdapter((self.context, request), name=name)
        if view is not None:
            return view

        raise NotFound(self.context, name, request)

    def traverse(self, name):
        # this will be overridden by subclasses
        pass


class ContextTraverser(Traverser):
    """Base class for context traversers in Grok applications.

    A context traverser is like a normal `grok.Traverser` but, instead
    of supplying its own `traverse()` method, it directs Grok to go call
    the ``traverse()`` method on the context itself in order to process
    the next name in the URL.

    """
    component.adapts(interfaces.IContext, IHTTPRequest)

    def traverse(self, name):
        traverse = getattr(self.context, 'traverse', None)
        if traverse:
            return traverse(name)


class ContainerTraverser(Traverser):
    """Base class for container traversers in Grok applications.

    A container traverser is like a normal `grok.Traverser` but, instead
    of supplying its own ``traverse()`` method, Grok will either call
    the ``traverse()`` method on the context itself, if any, else call
    ``get()`` on the container (a getitem-style lookup) in order to
    resolve the next name in the URL.

    """
    component.adapts(interfaces.IContainer, IHTTPRequest)

    def traverse(self, name):
        traverse = getattr(self.context, 'traverse', None)
        if traverse:
            result = traverse(name)
            if result is not None:
                return result
        # try to get the item from the container
        return self.context.get(name)


class IndexesClass(object):
    """Base class for index collections in a Grok application.

    A `grok.Indexes` utility provides one or more Zope Database content
    indexes for use in a `grok.Site` or `grok.Application`.  The site or
    application that the indexes are intended for should be named with
    the `grok.site()` directive, and the kind of object to index should
    be named with a `grok.context()` directive.

    Inside their class, the developer should specify one or more
    `grok.index.Field` instances naming object attributes that should be
    indexed (and therefore searchable)::

        class ArticleIndex(grok.Indexes):
            grok.site(Newspaper)
            grok.context(Article)
            author = index.Field()
            title = index.Field()
            body = index.Text()

    See the `grok.index` module for more information on field types.

    Note that indexes are persistent: they are stored in the Zope
    database alongside the site or application that they index.  They
    are created when the site or application is first created, and so an
    already-created site will not change just because the definition of
    one of its `grok.Indexes` changes; it will either have to be deleted
    and re-created, or some other operation performed to bring its
    indexes up to date.

    """
    def __init__(self, name, bases=(), attrs=None):
        if attrs is None:
            return
        indexes = {}
        for name, value in attrs.items():
            # Ignore everything that's not an index definition object
            # except for values set by directives
            if '.' in name:
                setattr(self, name, value)
                continue
            if not interfaces.IIndexDefinition.providedBy(value):
                continue
            indexes[name] = value
        self.__grok_indexes__ = indexes
        # __grok_module__ is needed to make defined_locally() return True for
        # inline templates
        self.__grok_module__ = martian.util.caller_module()

Indexes = IndexesClass('Indexes')


class Role(securitypolicy_Role):
    """Base class for roles in Grok applications.

    A role is a description of a class of users that gives them a
    machine-readable name, a human-readable title, and a set of
    permissions which users belong to that role should possess::

        class Editor(grok.Role):
            grok.name('news.Editor')
            grok.title('Editor')
            grok.permissions('news.EditArticle', 'news.PublishArticle')

    """



"""
An annotations class must implement either exactly one interface, or
it should cspecify which of the many implemented interfaces it should
be registered for.  Ambiguities lead to errors:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
  GrokError: <class
  'grok.tests.annotation.implementsmany.MammothAnnotations'> is
  implementing more than one interface (use grok.provides to specify
  which one to use).
"""

import grok
from zope import interface

class Mammoth(grok.Model):
    pass

class IOneInterface(interface.Interface):
    pass

class IAnotherInterface(interface.Interface):
    pass

class MammothAnnotations(grok.Annotation):
    grok.implements(IOneInterface, IAnotherInterface)



"""
We now demonstrate the use of a SetIndex with Grok::

Let's set up a site in which we manage a couple of objects::

  >>> herd = Herd()
  >>> getRootFolder()['herd'] = herd
  >>> from zope.app.component.hooks import setSite
  >>> setSite(herd)

Now we add some indexable objects to the site::

  >>> herd['alpha'] = Mammoth('Alpha', ['big', 'brown'])
  >>> herd['beta'] = Mammoth('Beta', ['big', 'black', 'friendly'])
  >>> herd['gamma'] = Mammoth('Gamma', ['brown', 'friendly', 'gorgeous'])

Let's query the set index::

  >>> from zope.app.catalog.interfaces import ICatalog
  >>> from zope.component import getUtility, queryUtility
  >>> catalog = getUtility(ICatalog)
  >>> def sortedResults(catalog, **kw):
  ...    result = list(catalog.searchResults(**kw))
  ...    result.sort(key=lambda x:x.name)
  ...    return [item.name for item in result]
  >>> sortedResults(catalog, features={'any_of': ['brown']})
  ['Alpha', 'Gamma']
  >>> sortedResults(catalog, features={'any_of': ['big']})
  ['Alpha', 'Beta']
  >>> sortedResults(catalog, features={'any_of': ['friendly']})
  ['Beta', 'Gamma']
  
Nuke the catalog and intids in the end, so as not to confuse
other tests::

  >>> sm = herd.getSiteManager()
  >>> from zope.app.catalog.interfaces import ICatalog
  >>> sm.unregisterUtility(catalog, provided=ICatalog)
  True
  >>> from zope.app.intid.interfaces import IIntIds
  >>> from zope import component
  >>> intids = component.getUtility(IIntIds)
  >>> sm.unregisterUtility(intids, provided=IIntIds)
  True

Unfortunately ftests don't have good isolation from each other yet.
"""

from zope.interface import Interface, Attribute
from zope import schema

import grok
from grok import index

class Herd(grok.Container, grok.Application):
    pass

class IMammoth(Interface):
    features = Attribute('Features')
    
class MammothIndexes(grok.Indexes):
    grok.site(Herd)
    grok.context(IMammoth)

    features = index.Set()

class Mammoth(grok.Model):
    grok.implements(IMammoth)

    def __init__(self, name, features):
        self.name = name
        self.features = features
    



"""
We define grokkers for the three base classes Alpha, Beta, and Gamma with different
priorities:
 
- AlphaGrokker with priority 0 (default)
- BetaGrokker with priority 1
- GammaGrokker with priority -1

    >>> import grok
    >>> grok.testing.grok(__name__)

We grok a module that implements subclasses for Alpha, Beta, and Gamma and our
grokkers get executed in the order of priority (highest first)::

    >>> grok.testing.grok('grok.tests.grokker.priority_fixture')
    beta
    alpha
    gamma

"""
import martian

class Alpha(object):
    pass


class Beta(object):
    pass

class Gamma(object):
    pass

class AlphaGrokker(martian.ClassGrokker):
    martian.component(Alpha)
    
    def grok(self, name, factory, module_info, **kw):
        print "alpha"
        return True

class BetaGrokker(martian.ClassGrokker):
    martian.component(Beta)
    martian.priority(1)
    
    def grok(self, name, factory, module_info, **kw):
        print "beta"
        return True
    
class GammaGrokker(martian.ClassGrokker):
    martian.component(Gamma)
    martian.priority(-1)
    
    def grok(self, name, factory, module_info, **kw):
        print "gamma"
        return True



"""
Mind that there is a subscriber to the containermodified event in order to
illustrate an ordered container fires events just like normal containers::

  >>> grok.testing.grok(__name__)
  >>> bones = OrderedBones()

Add an item::

  >>> bones['thigh'] = Bone('Thigh Bone')
  Container has changed!

Now change the order::

  >>> bones.updateOrder(order=['thigh'])
  Container has changed!

Delete an item::

  >>> del bones['thigh']
  Container has changed!
  >>> bones.keys()
  []

"""

import grok

class OrderedBones(grok.OrderedContainer):
    pass

class Bone(grok.Model):
    def __init__(self, name):
        self.name = name

from zope.app.container.interfaces import IContainerModifiedEvent
@grok.subscribe(OrderedBones, IContainerModifiedEvent)
def container_changed(object, event):
    print 'Container has changed!'



import grok
from zope import interface

class IFireplace(interface.Interface):
    pass

class Fireplace(grok.LocalUtility):
    grok.implements(IFireplace)

class Fireplace2(grok.LocalUtility):
    grok.implements(IFireplace)
    
class Cave(grok.Model, grok.Site):
    grok.local_utility(Fireplace, provides=IFireplace)
    grok.local_utility(Fireplace2, provides=IFireplace)



import grok

class Calculator(grok.GlobalUtility):
    grok.implements(ICalculator)  # if this is not specified, it breaks
    grok.name('')  # this is actually the default
    grok.provides(ICalculator) # this is actually the default

grok.global_utility(factory, provides=IFace, name=u'')

class Calculator(grok.LocalUtility):
    grok.utility_provides(ICalculator)

class Anything(grok.Model):
    pass

class NonPersistent(object):
    pass

class SpecialAnything(Anything):
    pass

class Foo(grok.Model, grok.Site):    
    grok.local_utility(Anything, hide=False, name_in_container='foo',
                       persistent=None)
    grok.local_adapter()
    grok.local_view()

class Foo2(Foo):
    grok.local_utility(SpecialAnything)

    



"""
Grok allows you to set up catalog indexes in your application with a
special indexes declaration. In fact, we have multiple grok.Indexes
setting up more than one set of indexes in the same catalog.

Let's set up a site in which we manage a couple of objects::

  >>> herd = Herd()
  >>> getRootFolder()['herd'] = herd
  >>> from zope.app.component.hooks import setSite
  >>> setSite(herd)

We are able to query the catalog::

  >>> from zope.app.catalog.interfaces import ICatalog
  >>> from zope.component import getUtility, queryUtility
  >>> catalog = getUtility(ICatalog)
  >>> sorted(catalog.keys())
  [u'age', u'age2', u'message', u'message2', u'name', u'name2']
  
Nuke the catalog and intids in the end, so as not to confuse
other tests::

  >>> sm = herd.getSiteManager()
  >>> from zope.app.catalog.interfaces import ICatalog
  >>> sm.unregisterUtility(catalog, provided=ICatalog)
  True
  >>> from zope.app.intid.interfaces import IIntIds
  >>> from zope import component
  >>> intids = component.getUtility(IIntIds)
  >>> sm.unregisterUtility(intids, provided=IIntIds)
  True

Unfortunately ftests don't have good isolation from each other yet.
"""

from zope.interface import Interface
from zope import schema

import grok
from grok import index

class Herd(grok.Container, grok.Application):
    pass

class IMammoth(Interface):
    name = schema.TextLine(title=u'Name')
    age = schema.Int(title=u'Age')
    def message():
        """Message the mammoth has for the world."""

class IMammoth2(Interface):
    name2 = schema.TextLine(title=u'Name')
    age2 = schema.Int(title=u'Age')
    def message2():
        """Message the mammoth has for the world."""

class MammothIndexes(grok.Indexes):
    grok.site(Herd)
    grok.context(IMammoth)

    name = index.Field()
    age = index.Field()
    message = index.Text()

class MammothIndexes2(grok.Indexes):
    grok.site(Herd)
    grok.context(IMammoth2)

    name2 = index.Field()
    age2 = index.Field()
    message2 = index.Text()



import grok

class Sample(grok.Application, grok.Container):
    pass

class Index(grok.View):
    def update(self, value1=None, value2=None):
        try:
            value1 = int(value1)
            value2 = int(value2)
        except (TypeError, ValueError):
            self.sum = "No sum"
            return
        self.sum = value1 + value2
